;***************************************************************************
;* Title: ATmega-TNC REV 1.8
;* Made available for educational and non-commercial use.
;* Commercial use forbidden without license from author.
;* Code may be modified as long as it is released publicly and credit
;* is given to the original author(s).  
;* (c) copyright 2003 Henry Carl Ott N2RVQ, all rights reserved.
;* Questions, comments or offers of cash? carlott@si.rr.com  http://users.rcn.com/carlott/
;**************************************************************************
;* Description: A basic TNC to provide UI connectivity for APRS type applications.
;* Command line interface to configure TNC for stand alone tracker operation or for use with PC based software.
;* Support for auxiliary half duplex 4800 baud GPS serial port, and rx only 2400 baud wx serial port. GPS data is accessible to host. 
;* Please read separate command file for complete list of configuration and operation commands.  
;**************************************************************************
;* Credits: The circular buffer routines were originally some very nice code from David VanHorn's terrific getting started
;* package, until I butchered the hell out of it.  Thanks to people at AVRFREAKS.COM and the atmel mail list. 
;* Also thanks to everybody who made their code available on-line for me to learn from. 
;***************************************************************************
;* Disclaimer: I'm a lousy programmer. Please don't use this code/design for any
;* application where human life is at risk or property damage may occur.
;***************************************************************************
;* REVISIONS:
;*1.8 bootloader support, lpath support, misc tweaks
;*1.73 minor tweaks in eng/gps support / echo bug fix	
;*1.72 added eng support for ui-view gps support (will send rmc string upon reception of 0x05)
;*1.71 added wdt support, misc tweaks
;*1.7 added wx serial port support 2400 baud rx only bit bang (uses timer and int)
;*1.61 added header and mcom commands (needed for ui-view compatibility)
;*1.6 converted aux rs-232 routine to use linear buffer, fix (maybe) overflow bug on 232 rcv 
;*1.5 changed command table, moved strings into flash,  fixed ssid bug on unproto digis  
;*1.4 fixed bug in aux isr handler, minor formatting changes 
;*1.3 smart(maybe) handling of cr-lf on incoming packets
;*1.2 trace, debug, lf, * on heard station, misc tweaks, bug fixes
;*1.1 minor changes, support slot time and persist for tx  
;*1.0 first release (cry havoc, release the bugs of war)

;****************************************************************************
;* Note: fuses ponyprog boxes checked  BOOTZS1 BOOTZS0 SUT0 BODLEVEL BODEN
;***************************************************************************
 
;--- start of the actual code ---
.nolist
.include "m8def.inc"
.list
.listmac

;***************************************************************************
;*
;* MACROS
;*
;***************************************************************************

;--- simple i/o macros
.macro	led1_on
	sbi	LED_PORT,LED1
.endm	
.macro	led1_off
	cbi	LED_PORT,LED1
.endm	

.macro	led2_on
	sbi	LED_PORT,LED2
.endm	
.macro	led2_off
	cbi	LED_PORT,LED2
.endm	

.macro	cts_on			; pc can talk
	cbi	CTS_PORT,CTS
.endm
		
.macro	cts_off			; pc should shut up
	sbi	CTS_PORT,CTS
.endm

.macro	relay1_close
	sbi	RELAY_PORT,K1
.endm
		
.macro	relay1_open
	cbi	RELAY_PORT,K1
.endm

;--- mx-614 specific macros 
.macro	_tx_mx614
	sbi	MX_PORT,M0
	cbi	MX_PORT,M1	
.endm
	
.macro	_rx_mx614
	cbi	MX_PORT,M0
	sbi	MX_PORT,M1	

.endm
	
.macro	_off_mx614		
	sbi	MX_PORT,M0	; power down mode
	sbi	MX_PORT,M1
.endm
	
;--- radio control
.macro	key_radio
	sbi	PTT_PORT,PTT
.endm

.macro	unkey_radio
	cbi	PTT_PORT,PTT
.endm	

;-------- rs-232 related macros -------------------------------------
;--------------------------------------------------------------------
;--- sends single char to console
.macro	_send
	ldi	temp,@0 
	rcall	putc
.endm	

;--- send ram string to console
.macro _ram2console
	ldi	zh,high(@0)
       	ldi	zl,low(@0) 	;Init Z-pointer
	rcall	ram_str2con
.endm

;--- send flash stored string to console
.macro	_send_flash
	ldi	zh,high(@0 * 2)
       	ldi	zl,low(@0 * 2) 	;Init Z-pointer
	rcall	flash_str2con
.endm

;----------------------------------------------------------
;--- skip next instruction
.macro	skip
		rjmp	(pc + 2)	; skip, next instruction	
.endm	

;--- load x pointer with arg
.macro _init_x
		ldi	xh,high(@0)
       		ldi	xl,low(@0) 	
.endm	

;--- load y pointer with arg
.macro _init_y
		ldi	yh,high(@0)
       		ldi	yl,low(@0) 	
.endm	

;--- load z pointer with arg
.macro _init_z
		ldi	zh,high(@0)
       		ldi	zl,low(@0) 	
.endm	


;--- load z pointer with arg*2 for flash stored data (convert byte to word pointer) 
.macro	_init_zflash
		ldi	zh,high(@0 * 2)
       		ldi	zl,low(@0 * 2) 	;Init Z-pointer
.endm	

;--- load eeaddr hi/lo with arg
.macro _init_eeaddr
		ldi	temp,low(@0)
		mov	eeaddr_l,temp
		ldi	temp,high(@0) 	
		mov	eeaddr_h,temp		
.endm	

;----- used for debugging
.macro _diag
		ldi	debug_val,@0
		rcall	debug_dump
.endm		

;**************************************************************************
;
;                                  ATmega8
;                                 _____ _____
;                                |     -     |
;                         *RST --| 1      28 |--  PC5   <- DET
;           232_RXD  ->   PD0  --| 2      27 |--  PC4	-> M0
;           232_TXD  <-   PD1  --| 3      26 |--  PC3   <- MX_RXD
;           AUX_RXD  ->   PD2  --| 4      25 |--  PC2   -> M1
;           WX_RXD   ->	  PD3  --| 5      24 |--  PC1	-> MX_TXD
;           AUX_TXD  <-   PD4  --| 6      23 |--  PC0	<- *INH
;                         Vcc  --| 7      22 |--  GND	 	
;                         GND  --| 8      21 |--  AREF   
;                       xtal1  --| 9      20 |--  AVCC	
;                       xtal2  --| 10     19 |--  PB5   
;                led1 <-  PD5  --| 11     18 |--  PB4          
;                led2 <-  PD6  --| 12     17 |--  PB3   
;                 cts <-  PD7  --| 13     16 |--  PB2   -> PTT                                                    
;                  K1 <-  PB0  --| 14     15 |--  PB1   <- strap                                                    
;                                |___________|
;
;**************************************************************************
;--- port and pin defines


;--- PORTB

.EQU	RELAY_PORT	= PORTB
.EQU	PTT_PORT	= PORTB

.EQU	K1		= 0x00	;O
.EQU	STRAP		= 0x01  ;O
.EQU	PTT		= 0x02  ;O 
 

;--- PORTD
.EQU WX_PORT		= PORTD
.EQU WX_INP		= PIND
		
.EQU CTS_PORT		= PORTD	 
.EQU LED_PORT		= PORTD
.EQU AUX_PORT 		= PORTD
.EQU AUX_INP 		= PIND
.EQU AUX_DDR 		= DDRD

.EQU	RS232_RXD	= 0x00	;I
.EQU	RS232_TXD	= 0x01	;O
.EQU	AUX_RXD		= 0x02	;I
.EQU    WX_RXD		= 0x03	;I
.EQU	AUX_TXD		= 0x04	;O
.EQU	LED1		= 0x05	;O
.EQU	LED2		= 0x06	;O
.EQU	CTS		= 0x07	;O

;--- PORTC 
.EQU INH_PORT		= PINC	
.EQU MX_PORT 		= PORTC
.EQU MX_INP 		= PINC
.EQU MX_DDR 		= DDRC


.EQU	INH		= 0x00  ;I
.EQU	MX_TXD		= 0x01	;O
.EQU	M1		= 0x02	;O
.EQU	MX_RXD		= 0x03	;I
.EQU	M0		= 0x04	;O
.EQU	DET		= 0x05	;I

;--- generic constants
;--- some ascii char codes
.equ	ctrl_c 		= 0x03
.equ	etx		= 0x03		 
.equ	eot		= 0x04
.equ	enq		= 0x05
.equ	bel 		= 0x07 		; bell
.equ	bs  		= 0x08		; back space
.equ	cr  		= 0x0d		; carriage return 
.equ	lf  		= 0x0a		; line feed

;--- timer reload values
.equ	T0_OFF		= 0x00  ;
.equ	T0_DIV1		= 0x01	;
.equ	T0_DIV8		= 0x02	;
.equ	T0_DIV64	= 0x03	;
.equ	T0_DIV256	= 0x04	;
.equ	T0_DIV1024	= 0x05	;
.equ	T0_EXT_FALL	= 0x06	;
.equ	T0_EXT_RISE	= 0x07	;

.equ	T1_OFF		= 0x00
.equ	T1_DIV1		= 0x01	;
.equ	T1_DIV8		= 0x02	;
.equ	T1_DIV64	= 0x03	;
.equ	T1_DIV256	= 0x04	;
.equ	T1_DIV1024	= 0x05	;
.equ	T1_EXT_FALL	= 0x06  ;
.equ	T1_EXT_RISE	= 0x07  ;

.equ	T2_OFF		= 0x00	;	
.equ	T2_DIV1		= 0x01	;
.equ	T2_DIV8		= 0x02	;
.equ	T2_DIV32	= 0x03	;
.equ	T2_DIV64	= 0x04	;
.equ	T2_DIV128	= 0x05	;
.equ	T2_DIV256	= 0x06	;
.equ	T2_DIV1024	= 0x07	;


;--- program specific constants	
.equ	clock 		= 7372800 	; cpu clock
.equ	console_baud 	= 9600		; default
.equ	aux_baud	= 4800		; for nmea strings
.equ	wx_baud		= 2400	
.equ	aux_bit_reload	= ((clock/64) / aux_baud)
.equ	wx_bit_reload	= ((clock/64) / wx_baud) 

.equ	isr_reload_val= (0xffff - (clock/9600)) + 21 ;(21=overhead)

.equ	head_flags 	= 40	; number of flag bytes before data
.equ	tail_flags 	= 2	; number of flags to send after data
.equ	tx_hang		= 1	; delay (.001) after last flag before un-keying

.equ	min_nmea_len	= 10	; minimun length of received nmea string (adjust later)
.equ	gprmc_checksum	= 0x4b  ; precalculated checksum of string 'GPRMC' (saves having to do it later)


.equ	min_packet_len 	= 17	; minimum length, + con+pid and crc of rx packet 
.equ	reload_count	= 59	; ax25 rxd transition time interval counter reload value, used by rx isr
.equ	control_byte	= 0x03	; control and pid for ax25 ui packets
.equ	pid_byte	= 0xf0	;	

.equ	rs232_tx_len	  = 96	; tx circ buffer
.equ	rs232_rx_len	  = 148	; input buffer length for console data  (128 bytes for data, and a few for pc fifo)
.equ	rs232_hs_hi	  = (rs232_rx_len - 16) 	; hi limit to buffer space to deassert cts line	and tell pc to be quiet
.equ	rs232_hs_lo       = (rs232_rx_len - 32)         ; low limit to asser cts and allow pc to talk
.equ	rs232_stall_cnt	  = 30	; length in seconds cts can be de-asserted. (comm is stalled) 	
.equ	max_ee_len	  = 48	; maximum length of eeprom stored strings, (ltext, gpsinit) includes a char for zero termination
.equ	max_btext_len	  = 64  ; maximum length of btext, including zero termination

.equ	aux_rx_len	  = 80	; aux_input buffer max size
.equ	wx_rx_len	  = 57	; wx input max string length	
.equ	max_str_len	  = 80	; workspace length
.equ	max_rx_packet_len = 190	; maximim length, determined by available ram, isr will discards extra chars
.equ	max_packet_data   = 128	; max size of packet payload, tx and rx
.equ	max_digis	  = 6	; maximum number of digis to support (allocates ram buffer n*7)
.equ	rmc_max_len  	  = 80		


;*****************************
;* Global Register Variables *
;*****************************
.def	zero		= r00	; always left as zero to simplify math and other operations , can use in isrs if cleared before reti

.def	ax25_txbuff 	= r01	; isr, ax25 byte data to send	
.def	ax25_tx_counts	= r02	; isr, bit counters (char bits and ones counter for bit stuffing)
.def	ax25_rx_interval= r03	; isr, time interval between rxd transitions
.def	ax25_rx_bitcnt  = r04	; isr, counts incoming bits to build a byte	
.def	ax25_rxbuff	= r05	; isr, incoming bits are assembled here to form bytes

.def	aux_uart_status	= r06	; bit counter, plus status bits
.def	aux_char_buff	= r07	; place to build  aux data rx, storage for aux xmit
.def	wx_uart_status	= r08	; bit counter, plus status bits
.def	wx_char_buff	= r09	; place to build wx data rx byte 
.def	cr_cnt		= r10	; inc evertime a cr is received from console 

.def	eeaddr_l	= r12	; eeprom address low byte, must be on even register boundry (for incs) 
.def	eeaddr_h	= r13	; eeprom address high byte		

.def	isr_save	= r15	; dedicated register used to save status during isr

.def	debug_val	= r16	; used for debugging
.def	temp		= r17	; gp working reg      ; anybody can trash	
.def	temp2		= r18	; another gp register ;   	
.def	temp3		= r19	; and another	      ;		
.def	ii		= r20	; gp loop counter

.def	sys_flags	= r21	; sys bit flags
.def	nv_flags	= r22	; copy of non-volitle operation flags, saved and restored from eeprom
.def	ax25_flags	= r23   ; bit flags for rx isr
.def	tracking_flags	= r24	; bits related to gps and tracking


;.def	XL		=r26
;.def	XH		=r27
;.def	YL		=r28
;.def	YH		=r29
;.def	ZL		=r30
;.def	ZH		=r31


;--- aux uart related bit flags (half duplex)
.equ	aux_cnt0	= 0x00		; bit counter	
.equ	aux_cnt1 	= 0x01	
.equ	aux_cnt2	= 0x02	
.equ	aux_cnt3	= 0x03 		

.equ	aux_txrx	= 0x06		; isr mode tx=1, rx=0
.equ	aux_busy	= 0x07		; byte transfer in progress

;--- wx uart related bit flags (half duplex)
.equ	wx_cnt0		= 0x00		; bit counter, 0-9
.equ	wx_cnt1 	= 0x01	
.equ	wx_cnt2		= 0x02	
.equ	wx_cnt3		= 0x03 		

.equ	wx_cr_rcvd	= 0x07		; a cr was received

;--- sys bit flags 
.equ	ctrl_c_rcvd	= 0x00		; ctrl_c received from console 
.equ	debug		= 0x01		; set debug mode, tx data is dumped
.equ	converse	= 0x02		; all cr terminated console strings are shipped out to radio
.equ	trace		= 0x03		; hex dump of packets
.equ	enq_rcvd	= 0x04		; an enq (0x05 was received on main serial port (and discarded) (send gps string)
.equ	monitor		= 0x05		; display packets enable
.equ	_1hz		= 0x06		; set every second by isr
;--- non volitile bit flags

.equ	echo		= 0x00 		; console echo
.equ	pwrupconv	= 0x01		; 1= fall into converse mode on pwr up
.equ	lf_en		= 0x02		; if set append lf to end of console lines	
.equ	header		= 0x03		; if set put a cr between header and data (mainly for uiview compatibility)
.equ	mcom		= 0x04		; display packet type                     (mainly for uiview compatibility)
.equ	ax_lf_en	= 0x05		; if set append linefeeds to transmitted ax25 packets

;--- tracking and aux serial port related bit flags
.equ	rmc_rcvd	= 0x00		; a good rmc string was received (at least the checksum matched)
.equ	tracking	= 0x01		; we have valid tracking! ('A' in rmc string)
.equ	aux_cr_rcvd	= 0x02		; cr on aux port received

;--- ax25 rx/tx bit flags		; isr
.equ	rx_enable	= 0x00		; enable reception of packet data
.equ	in_packet 	= 0x01		; currently receiving packet data (we are between flags)
.equ	flag_det	= 0x02		; the last byte received was a valid flag	
.equ	bit_sample	= 0x03		; last sample of input, must be in same bit location as input pin (MX_INP) 
.equ	packet_rx	= 0x04		; a valid packet was received and is sitting in the _ax25_rx_buffer	
.equ	over_run	= 0x05		; the start of a packet before last packet was processed (removed from buffer)	
.equ	tx_bit_stuff	= 0x06		; enable bit stuffing on ax25 tx 


;--- ax25_tx_counts			; isr
.equ	tx_cnt0		= 0x00		; bit counter for ax25 tx routine, 0 = no bits to send 
.equ	tx_cnt1		= 0x01		; 
.equ	tx_cnt2		= 0x02
.equ	tx_cnt3		= 0x03	

.equ	ones_cnt0	= 0x04		; ax25 tx ones counter (used for but stuffing)
.equ	ones_cnt1	= 0x05
.equ	ones_cnt2	= 0x06


.DSEG


;--- keep 16 bit timers in order!
_rmc_exp_timer:		.byte	2		; expire rmc sentence
_btimer:		.byte	2		; beacon timer 	
_ltimer:		.byte	2		; send location timer 

_rs232_timer:		.byte	1		; used to check for stalled conditions on console port
_tic:			.byte	1		; dec at 9600 hz 95-00
_ms_10:			.byte	1		; dec at 100hz 99-00


_work_buff:		.byte	max_str_len	; general purpose workspace						;- keep timers in order!
_232_rx_buffer:		.byte	rs232_rx_len+3	; size,head,tail, data    
_232_tx_buffer:		.byte	rs232_tx_len+3

_random:		.byte	1		; storage fore 8 bit random routine

_tx_crc_hi:		.byte	1		; storage for tx crc routine
_tx_crc_lo:		.byte	1	

_rx_crc_hi:		.byte	1		; storage for rx crc routine
_rx_crc_lo:		.byte	1	

_ax25_rx_cnt:		.byte	1		; number of received chars in buffer
_ax25_rx_buffer:	.byte	max_rx_packet_len	; max size of buffer (raw data)

_ax25_dest_call:	.byte	7		; destination 
_ax25_from_call:	.byte	7		; source (source & dest locations ARE linked!)

_ax25_ctrl:		.byte 	1		; control byte of last received packet
_ax25_pid:		.byte	1		; PID of last received byte

_ax25_digi_cnt:		.byte 	1		; number of chars in digi path 
_ax25_digi_buffer:	.byte	max_digis*7	; digis are 7 chars each

_ax25_rxtx_cnt:		.byte	1		; num of chars in packet payload
_ax25_rxtx_buffer:	.byte	max_packet_data	; contains data payload, buffer used for tx and rx

_wx_rx_cnt:		.byte	1		; points to next char to store	
_wx_rx_buffer:		.byte	wx_rx_len	; linear buffer for aux serial port

_rmc_cnt:		.byte	1		; chars in rmc string (no cr)
_rmc_buffer:		.byte	rmc_max_len	; zero terminated storage of rmc string

_aux_rx_cnt:		.byte	1		; byte count	
_aux_rx_buffer:		.byte	aux_rx_len	; linear buffer for aux serial port

_debug_temp:		.byte	1		; place to store temp register for debug routine
;_ram_mark:		.byte	1		; just a marker to show me the last byte in lst file, can be removed

;-----------------------------------------------
;--- start of code space
.CSEG
.org		0x00
		rjmp	reset			; Reset Handle	
.org		INT0addr			; external int 0 
		rjmp	int0_isr		; aux gps 4800 baud inp	(bit banged)
.org		INT1addr			; external int 1
		rjmp	int1_isr		; wx 2400 data inp (bit banged)
.org		OVF0addr			; overflow of 8  bit timer 0 
		rjmp	t0int_isr		; timer used for gps nmea data recetion
.org		OVF1addr			; overflow of 16 bit timer 1 
		rjmp	t1int_isr		; basic 9600 hz ax25 rx/tx routines	
.org		OVF2addr			; overflow of 8 bit timer 2 
		rjmp	t2int_isr		; timer used for wx 2400 data reception	
.org 		URXCaddr			; reception of data by uart
		rjmp	uart_rx_isr
.org		UDREaddr			; uart tx buffer is empty
		rjmp	udr_int

.org		SECONDBOOTSTART
.include	"at-tnc_bootloader.asm"		; bootloader support (loads at  SECONDBOOTSTART)

.org		0x13				; first location after reset vectors
.include	"at-tnc_timer.asm"
.include	"at-tnc_uarts.asm"

;-----------------------------------------------------------------------
;------------------------------------------------------------------------
;--- start here on power up, external reset, brownout, or wdt reset
;--- init ports, ints, and  zero all registers, and ram 		
reset:		ldi	temp,0
		_init_z	0x00				; bottom of registers	

clr_reg:	st	z+,temp
		cpi	zl,0x1e				; don't zero z reg
		brne	clr_reg
	
		_init_z	0x60				; beginning of ram

clr_ram:	st	z+,r16
		cpi	zl,LOW(RAMEND) 
		brne	clr_ram		
		cpi	zh,HIGH(RAMEND)
		brne	clr_ram	 
		
	
;--- init stack pointer (somewhat important)
		ldi	temp,LOW(RAMEND) 	;setup stack pointer
		out 	SPL,temp
		ldi	temp,HIGH(RAMEND)
		out	SPH,temp

;--- PORTB setup	
		ldi	temp,(1<<K1)|(1<<PTT)
		out	DDRB,temp
		ldi	temp,(1<<PB5)|(1<<PB4)|(1<<PB3)|(1<<STRAP) ; pull ups on unused pins.	
		out	PORTB,temp	

;--- PORTC setup	; mx_614
		ldi	temp,(1<<MX_TXD)|(1<<M0)|(1<<M1)
		out	DDRC,temp				; set outputs
		ldi	temp,1<<inh				
		out	PORTC,temp				; all low

;--- PORTD setup
		ldi	temp,(1<<RS232_TXD)|(1<<AUX_TXD)|(1<<CTS)|(1<<LED1)|(1<<LED2)
		out	DDRD,temp
		ldi	temp,(1<<WX_INP)			; pull up  
		out	PORTD,temp

;*****************************************************************
;--- setup uart, 9600 baud, int on rx, no tx ints
		;ldi	temp,(clock - 8 * console_baud) / (16 * console_baud)

		_init_zflash _baud_ubbr	
		_init_eeaddr _baud_ee				; get baud rate divider from eeprom
		rcall	eerd
		andi	temp,0x07	
		add	zl,temp
		adc	zh,zero
		lpm	temp,z
		
		out	UBRRL,temp				; Set baud rate generator
		ldi 	temp,(1<<URSEL)|(1<<UCSZ0)|(1<<UCSZ1)
		out 	UCSRC,temp
		ldi	temp,(1<<RXCIE)|(1<<RXEN)|(1<<TXEN) 	; enable rxc interrupts
		out	UCSRB,temp				; 

;*******************************************************************
;--- set up timer 0 prescaler 
		
		ldi	temp,T0_DIV64			;  
		out	TCCR0,temp			; timer0 prescale by 64	

;*******************************************************************
;--- set up timer 1  
		ldi	temp,T1_DIV1		; timer1 div by 0 prescaler
		out	TCCR1B,temp

		ldi	temp,(1<<TOIE1) 	; timer 1 overflow int
		out	TIMSK,temp

;*******************************************************************
;--- set up timer 2 prescaler
		ldi	temp,T2_DIV64
		out	TCCR2,temp
	
;*******************************************************************
;--- global interupts enable
		sei

;----------------------------------------------------------------------------
;--- enable watch dog timer
	
		ldi	temp,(1<<WDCE)|(1<<WDE)
		out	WDTCR,temp					; enable changes to wdt
		
		ldi	temp,(1<<WDP2)|(1<<WDP1)|(1<<WDP0)|(1<<WDE)	; turn on watch dog time, longest timeout
		out	WDTCR,temp

;--- init regs and vars

		;clr	zero
		ldi	temp,0xaa
		sts	_random,temp		; init random number generator (won't start if zero)

		_off_mx614			; turn off mx614, 

		_init_eeaddr _nv_flags_ee	; get our non volatile flags from eeprom
		rcall	eerd
		mov	nv_flags,temp		; save in our runtime register

		sbrc	nv_flags,pwrupconv	; see if we are in power up converse mode
		sbr	sys_flags,1<<converse	; set converse mode	

;------------------------------------------------
; 		
		rcall	en_aux_rx			; enable half duplex software driven aux serial port
		rcall	en_wx_rx			; enable 2400 baud wx input port
		rcall	gps_init			; if we have a string in eeprom, send to aux port to init gps receiver
		
		cts_on					; assert cts line to pc

		_rx_mx614				; receive mode 
		sbr	ax25_flags,1<<rx_enable 	; enable rx isr
		sbr	sys_flags, 1<<monitor		; enable display of packets


		


disp_ver:	_send_flash _hello			; say hello to the world

prompt:		sbrc	sys_flags,converse		; if converse, don't display prompt
		rjmp	main_lp

		rcall	crlf
		_send_flash _cmd			; send cmd prompt to console		
		
main_lp:	wdr					; feed the dog
		rcall	get_random			; keep rolling the dice

		sbrc	ax25_flags,packet_rx		
		rcall	process_rcvd_packet		; display rcvd packet

		sbrc	ax25_flags,over_run
		rcall	over_run_error			; display error
		
		sbrc	tracking_flags,aux_cr_rcvd	
		rcall	get_rmc				; process aux string
	
		sbrc	wx_uart_status,wx_cr_rcvd
		rcall	ax_send_wx			; process wx string

		sbrc	sys_flags,enq_rcvd		; was an enq char received?
		rcall	gps_enq				; yes, send rmc string (if we have one)

		sbrc	sys_flags,ctrl_c_rcvd
		rcall	clr_converse			; handle ctrl-c char

		tst	cr_cnt				; any strings received?	
		brne	proc_c1

		sbrc	sys_flags,_1hz
	 	rcall	timer_events			; handle timer events (beacon, position, rs-232stall test)	

		rjmp	main_lp	
		
proc_c1:	sbrs	sys_flags,converse		; 
		rjmp	process_cmd			; if not in converse mode, process command
		rcall	ax_send_str			; if in converse mode, send string	
		rjmp	main_lp

process_cmd:	rcall	get232_str2work			; get cr terminated string from circ 232-buffer into null terminated work space (cr will be tossed)
		tst	temp
		breq 	prompt				; if zero length string 	
		rcall	get_cmd_addr			; convert into routine address
		ijmp					; now go do it!

cmd_err:	rcall	crlf				; will jump here if command does not exist
		_send_flash _eh
		rjmp	prompt	


;***********************************************************************
;--- 1 second timer events, called from main lp
;--- check for rs232 stalled, beacon , location tx
;***********************************************************************

timer_events:	cbr	sys_flags,1<<_1hz		; clear flag
		lds	temp,_rs232_timer		; get 232 stall timer
		cpi	temp,rs232_stall_cnt		; compare to limit
		brlo	rmc_exp_check	
		inc	cr_cnt				; dummy cr to get data out of buffer
		sts	_rs232_timer,zero		; reset counter

;--- check to see if gps sentence has expired
rmc_exp_check:	_init_eeaddr _rmc_expire_ee		; get threshhold
		_init_z	_rmc_exp_timer
		rcall	cmp_ee_timer			; compare it
		brlo	bcheck				; no timout or timer is shut off

		cbr	tracking_flags,(1<<rmc_rcvd) | (1<<tracking)	; 	
		sts	_rmc_exp_timer,zero				; reset timer  
		sts	_rmc_exp_timer+1,zero
		led2_off				; indicator
		
;--- see if we should beacon
bcheck:		_init_eeaddr	_btime_ee		; should be pointing at btimer 
		_init_z	_btimer
		rcall	cmp_ee_timer
		brlo	lcheck	
		rcall	ax_send_beacon
							
;--- see if we should send our location
lcheck:		_init_eeaddr	_ltime_ee	
		_init_z	_ltimer
		rcall	cmp_ee_timer
		brlo	tevents_end	
		rcall	ax_send_posit
tevents_end:	ret


;--- compare 16 bit val pointed by z to ee stored value
;--- set carry if timer is turned off or if z is lower the the eevalue
cmp_ee_timer:	rcall	eerd_p
		mov	yl,temp		; get low
		rcall	eerd_p
		mov	yh,temp		; get high

		sbiw	yl,0
		sec			; fail or off
		breq	cmp_ee_t_end	; beacon is turned off	

		ld	temp,z+
		cp	temp,yl
		ld	temp,z+
		cpc	temp,yh

cmp_ee_t_end:	ret	


;--- remove ctr-c from buffer / switch mode
clr_converse:	rcall	getc_232
		brcc	clr_conv_end		; error, buffer was empty	
		cpi	temp,ctrl_c		; keep removing chars till we get to ctrl-c
		brne	clr_converse	
			
clr_conv_end:	cbr	sys_flags,1<<converse | 1<<ctrl_c_rcvd
		rjmp	prompt


;***********************************************************************
;--- console commands, indexed jumps from main loop 
;***********************************************************************
;--- change baudrate, turn off ints jump to bootloader code, only way back is reset
enter_bootloader:
		cli
		ldi 	temp,(clock - 8 * bootldr_baud) / (16 * bootldr_baud) 	; Baud rate 
		out	UBRRL,temp
		rjmp	bootloadermain							; 

;--- restore eeprom contents to default values, fall into reset
restore_tnc:	rcall	restore_ee_2_defaults

;--- loope until watch dog timeout
reset_tnc:	rjmp	reset_tnc	

;--- dump registers and first ram bank
dump:		rcall	debug_dump
		rjmp	prompt

;--- relay 1 close or open 
k1_routine:	rcall	arg_count

		cpi	temp,02
		brne	disp_relay1

		ldi	temp,2				; point to second argument
		rcall	on_off				; check it

		brcc	pc + 2
		relay1_close 	

		brcs	pc + 2
		relay1_open 	
		
disp_relay1:	rcall	crlf				; display status
		_send_flash _k1
		rcall	send_eq	

		sbic	RELAY_PORT,K1	
		rjmp	disp_on
		rjmp	disp_off	

;--- enable converse mode
set_converse:	_send_flash _converse
		rcall	crlf
		sbr	sys_flags,1<<converse
		rjmp	main_lp

;--- send gps init string out aux port
send_gps_init:	rcall	gps_init
		rjmp	prompt

;------------------------------------------------------------------------
;--- if we have a valid (good crc) gps string in buffer, send it to console
;--- if no data send empty string 
gps_query:	rcall	gps_send_con		; called from cmd mode	
		rjmp	prompt

gps_enq:	rcall	gps_send_con		; send any data	
		cbr	sys_flags,1<<enq_rcvd	; clear flag
		ret	

gps_send_con:	sbrs	tracking_flags,rmc_rcvd	; see if we have a string in buffer	
		rjmp	gpsq_end

		_init_z	_rmc_buffer		; point to data
		lds	temp2,_rmc_cnt		; get count
		rcall	ram2con

gpsq_end:	rcall	crlf
		ret	

;------------------------------------------------------------------------
;--- formated list all command strings to console, (a minimalist help)
disp_commands:	_init_zflash _cmd_table
		ldi	temp2,0xff

disp_cmds_lp:	lpm	temp,z			; test for end of table
		tst	temp
		breq	disp_cmds_end

		cpi	temp2,0x40		; max 40 chars per line	
		brlo	disp_fill

		clr	temp2
		rcall	crlf
		rjmp	disp_cmd_lp2		

disp_fill:	ldi	temp,0b00001111		; attempted formatting
		and	temp,temp2
		breq	disp_cmd_lp2
		_send	' '
		inc	temp2
		rjmp	disp_fill	

		lpm	temp,z+			; send that non zero char		
		inc	temp2

disp_cmd_lp2:	lpm	temp,z+			;  
		tst	temp			; test for zero terminator, get next cmd
		breq	next_cmd 		;  

		rcall	putc			;  output char 
		inc	temp2
		rjmp	disp_cmd_lp2		;  and loop for next

next_cmd:	sbrc	zl,0			; point past table address	
		adiw	zl,1			; keep in mind byte boundries		
		adiw	zl,2		
		rjmp	disp_cmds_lp

disp_cmds_end:	rjmp	prompt

;------------------------------------------------------------------
;--- tx test and calibration routines 
;--- H = txd-hi    L = txd-low   A = 010101010 (@ 1200hz)  ctrl-c = exit
;--- K= key radio  U = unkey radio
cal_routines:	wdr								
		cbr	ax25_flags,1<<rx_enable
		_tx_mx614			; set mx-614 to tx mode

		rcall	getc_232
		brcc	cal_routines

cal_routine_a:	rcall	to_upper
		cpi	temp,ctrl_c
		brne	cal_key

		unkey_radio
		_rx_mx614
		sbr	ax25_flags,1<<rx_enable		
		rcall	flush232_rx_buff
		rjmp	prompt

cal_key:	cpi	temp,'K'					; ptt
		brne	cal_unkey	
		key_radio
		
cal_unkey:	cpi	temp,'U'					; un-key
		brne	send_hi
		unkey_radio

send_hi:	cpi	temp,'H'					; pin high
		brne	send_low
		sbi	MX_PORT,MX_TXD
		
send_low:	cpi	temp,'L'					; pin low
		brne	send_alt
		cbi	MX_PORT,MX_TXD

send_alt:	cpi	temp,'A'
		brne	cal_routines					; no match, keep looping
		
send_alt_lp:	wdr
		cbr	ax25_flags,(1<<tx_bit_stuff)			; turn off bit stuffing
		ldi	temp,0xaa
		rcall	putc_ax
		rcall	getc_232					; any key will exit
		brcs	cal_routine_a
		rjmp	send_alt_lp

;-----------------------------------------------------------------------------------
;--- serial port debugging
;--- send data back and forth between console and aux/wx ports.   ctrl-c = back to command mode
;--- cr terminated strings from console goes to the aux port and vice versa
;--- aux port is half duplex!  
;--- ignore ax25 rcv data (where would it go?)
aux_link:	wdr
		cbr	ax25_flags,1<<rx_enable			; turn off AX25 reception
	
		tst	cr_cnt
		breq	pc + 2
		rcall	rs2aux
		
		sbrc 	tracking_flags,aux_cr_rcvd		
		rcall	aux2rs		

		sbrc	wx_uart_status,wx_cr_rcvd
		rcall	wx2rs

		sbrc	sys_flags,ctrl_c_rcvd
		rjmp	flush_exit
		rjmp	aux_link		

rs2aux:		rcall	get232_str2work		; copy string to work space
		_init_z	_work_buff		; point to data
		
rs2aux_lp:	ld	temp,z+			; get a char
		tst	temp
		breq	aux_crlf		
		rcall	putc_aux	
		rjmp	rs2aux_lp		; send it


aux_crlf:	ldi	temp,cr			; send crlf out aux port
		rcall	putc_aux
		ldi	temp,lf
		rcall	putc_aux		
		ret

wx2rs:		_send_flash _wx			; show source
		_init_z	_wx_rx_cnt		; point to buffer
		ld	temp2,z+		; get pointer, bump z to data			
		rcall	ram2con
		rcall	flush_wx_buff	
		rcall	crlf	
		ret
		
aux2rs:		_send_flash _gps
		_init_z	_aux_rx_cnt		; point to buffer
		ld	temp2,z+		; get poiner, bump z to data	
		rcall	ram2con

		rcall	flush_aux_buff	
		rcall	crlf			; terminate it
		ret

flush_exit:	rcall	flush_wx_buff
		rcall	flush_aux_buff
		rcall	flush232_rx_buff
		sbr	ax25_flags,1<<rx_enable
		rjmp	prompt

;----------------------------------------------------
;--- get aprs position report symbols from console
get_symbol:	rcall	arg_count
		cpi	temp,03			; if not three arguments just display
		brne	disp_symbols

		_init_eeaddr _pos_table_char	; point to storage

		ldi	temp,2
		rcall	set_z2arg		; get first symbol
		rcall	eewr_p

		ldi	temp,3			; second sym
		rcall	set_z2arg	
		rcall	eewr_p
		
disp_symbols:	rcall	crlf			; display with pretty formatting
		_send_flash _symbol
		rcall	send_eq

		rcall	send_quote		; send quote symbol	
		_init_eeaddr _pos_table_char	
		rcall	eerd_p
		rcall	putc
		rcall	send_quote		; send quote symbol '''
		_send	' ' 
		rcall	send_quote		; send quote symbol
		rcall	eerd
		rcall	putc
		rcall	send_quote		; send quote symbol
		rjmp	prompt


;-------------------------------------------
;--- get strings from console save in eeprom
;--- get beacon text and save in eeprom 
get_btext:	ldi	xh,high(_btext*2)	 ; pointer to display string
		ldi	xl,low(_btext*2)
		_init_y _btext_ee		 ; pointer to storage	
		ldi	temp2,(max_btext_len - 1)		
		rjmp	get_ee_string + 1

;--- get gps init string
get_gpsistr:	ldi	xh,high(_gpsistr*2)
		ldi	xl,low(_gpsistr*2)
		_init_y _gpsinit_text_ee
		rjmp	get_ee_max		; limit chars	

;--- get comment field for posit reports
get_ltext:	ldi	xh,high(_ltext*2)
		ldi	xl,low(_ltext*2)
		_init_y _ltext_ee
get_ee_max:	ldi	temp2,(max_ee_len-1)	; maximum length of eestrings, less space for zero
		;rjmp	get_ee_string

;--- x points to msg string, y points to eeprom storage r2 max number of chars to copy
get_ee_string:	rcall	arg_count
		cpi	temp,01
		breq	disp_ee_string

		ldi	temp,2
		rcall	set_z2arg	

		cpi	temp,'%'		; see if first char is %. set2arg returns first char of arg
		brne	disp_ee_a

		rcall	arg_len			; check for single char
		cpi	temp,0x01	
		brne	pc + 2		
		st	z,zero

disp_ee_a:	movw	eeaddr_h:eeaddr_l,yh:yl
		rcall	ram_str2ee		; r2 (max length should be set)

disp_ee_string:	rcall	crlf
		movw	zh:zl,xh:xl
		rcall	flash_str2con
	
		rcall	send_eq			; send equals sign
		rcall	send_quote		; send quote symbol	
		
		movw	eeaddr_h:eeaddr_l,yh:yl
		rcall	ee2con
		rcall	send_quote		; send quote symbol
		rjmp	prompt			; back to top

;-------------------------------------------------------------------------
;--- get 8bit int from console and store in eeprom
get_fixtype:	ldi	xh,high(_fixtype*2)	; type of position report aprs or rmc 
		ldi	xl,low(_fixtype*2)
		_init_y _fix_type_ee
		rjmp	get_int2ee

get_txdelay:	ldi	xh,high(_txdelay*2)	; type of position report aprs or rmc 
		ldi	xl,low(_txdelay*2)
		_init_y _tx_delay_ee
		rjmp	get_int2ee

get_slot:	ldi	xh,high(_slot*2)	; get tx slot time
		ldi	xl,low(_slot*2)
		_init_y _slot_ee	
		rjmp	get_int2ee

get_persist:	ldi	xh,high(_persist*2)	; get p-persist val
		ldi	xl,low(_persist*2)
		_init_y _persist_ee
		rjmp	get_int2ee

get_baud:	ldi	xh,high(_baud*2)	; get baud number 0-7 see table elsewhere
		ldi	xl,low(_baud*2)
		_init_y _baud_ee	
		;rjmp	get_int2ee			

get_int2ee:
;--- x points to msg string
;--- y points to eeprom storage
		movw	eeaddr_h:eeaddr_l,yh:yl
		rcall	arg_count
		cpi	temp,02
		brne	disp_eeint

		ldi	temp,2
		rcall	set_z2arg
		rcall	asc2bin			; returns in z
	
		tst	zh			; size check
		breq	pc + 2	
		rjmp	cmd_err			; > 255
		
		mov	temp,zl			; get low
		rcall	eewr
		
disp_eeint:	rcall	crlf

		movw	zh:zl,xh:xl
		rcall	flash_str2con

		rcall	send_eq
		rcall	eerd

		mov	xl,temp
		clr	xh
		rcall	bin2console
		rjmp	prompt


;-------------------------------------------------------------------------
;--- get 16 bit value from console and store in eeprom, update ram counter

;--- get rmc sentence expire value
get_rmc_expire:	ldi	xh,high(_rmcexpire*2)
		ldi	xl,low(_rmcexpire*2)
		_init_eeaddr _rmc_expire_ee
		rjmp	get_eetimer

;--- get pfix (posit fix) time in seconds
get_ltime:	ldi	xh,high(_ltime*2)
		ldi	xl,low(_ltime*2)
		_init_eeaddr _ltime_ee
		rjmp	get_eetimer

;--- get beacon timer in seconds
get_btime:	ldi	xh,high(_beacon*2)
		ldi	xl,low(_beacon*2)
		_init_eeaddr _btime_ee
		;rjmp	get_eetimer

get_eetimer:
;---  x = msg pointer
;--- eeaddr points to eeprom storage, low,high 
		
		rcall	arg_count
		cpi	temp,01
		breq	disp_eetimer

		cpi	temp,4
		brsh	disp_eetimer	

;		ldi	temp,2			; if 2 or 3 args, get last one on line
		rcall	set_z2arg
		rcall	asc2bin			; returns in z

		mov	temp,zl			; get high
		rcall	eewr_p

		mov	temp,zh			; get low
		rcall	eewr
		
		dec	eeaddr_l		; dec ee pointer to original address
						; we don't handle paging!
disp_eetimer:	rcall	crlf

		movw	zh:zl,xh:xl		; get msg pointer
		rcall	flash_str2con
		rcall	send_eq			; send equals sign
		rcall	eerd_p			; get low byte
		mov	xl,temp

		rcall	eerd			; get high
		mov	xh,temp

		clr	temp
		or	temp,xh
		or	temp,xl		
		brne	pc + 2			; check for zero
		rjmp	disp_off

		rcall	bin2console		; else display value	
		_send	' '
		_send_flash _seconds		; 
		rjmp	prompt			; to top


;------------------------------------------------------
;--- get system bit flags, system flags don't get stored in eeprom

; get and set trace bit flag
get_trace:	ldi	xh,high(_trace*2)
		ldi	xl,low(_trace*2)
		ldi	yl,1<<trace
		rjmp	get_sys_flag

;--- get and set debug bit from console
get_debug:	ldi	xh,high(_debug*2)
		ldi	xl,low(_debug*2)
		ldi	yl,1<<debug
		rjmp	get_sys_flag
		
;--- get monitor status and save
get_monitor:	ldi	xh,high(_monitor*2)
		ldi	xl,low(_monitor*2)
		ldi	yl,1<<monitor
;		rjmp	get_sys_flag			

;--- get  sys  flag bits and update/store
;--- x = display message,  yl = bit flag mask
get_sys_flag:	rcall	arg_count
		cpi	temp,02
		brne	disp_sys_flag

		com	yl
		and	sys_flags,yl			; turn off bit in register
		com	yl

		ldi	temp,2				; point to second argument
		rcall	on_off				; check it
	
		brcc	pc + 2
		or	sys_flags,yl			; set it, 	

disp_sys_flag:	rcall	crlf				; display formatting	

		movw	zh:zl,xh:xl			; msg pointer
		rcall	flash_str2con

		rcall	send_eq				; make pretty	
		
		and	yl,sys_flags			; bit on or off?
		brne	disp_on			
		rjmp	disp_off


;------------------------------------------------------
;---- get non volitile flag bits, and store in eeprom, also update nv register
;--- append line feeds to console strings
set_lf:		ldi	xh,high(_lf*2)
		ldi	xl,low(_lf*2)
		ldi	yl,1<<lf_en
		rjmp	get_nv_flag

;--- if set append line feeds to ax25 packets
set_axlf:	ldi	xh,high(_axlf*2)
		ldi	xl,low(_axlf*2)
		ldi	yl,1<<ax_lf_en
		rjmp	get_nv_flag

;---- when set, tnc will powerup in converse mode
set_pwrup_conv_mode:
		ldi	xh,high(_pwrupconv*2)
		ldi	xl,low(_pwrupconv*2)
		ldi	yl,1<<pwrupconv
		rjmp	get_nv_flag

;--- get and set echo status from console
set_echo:	ldi	xh,high(_echo*2)
		ldi	xl,low(_echo*2)
		ldi	yl,1<<echo
		rjmp	get_nv_flag	
;--- get and set header bit (cr between header and data)
set_header:	ldi	xh,high(_header*2)
		ldi	xl,low(_header*2)
		ldi	yl,1<<header
		rjmp	get_nv_flag	

;--- get and set mcom bit (display packet type)
set_mcom:	ldi	xh,high(_mcom*2)
		ldi	xl,low(_mcom*2)
		ldi	yl,1<<mcom
		;rjmp	get_nv_flag	


;--- x = display message pointer,  yl = bit flag mask
get_nv_flag:	rcall	arg_count
		cpi	temp,02
		brne	disp_nv_flag

		com	yl
		and	nv_flags,yl			; turn off bit in register
		com	yl

		ldi	temp,2				; point to second argument
		rcall	on_off				; check it
	
		brcc	pc + 2
		or	nv_flags,yl			; set it, 	

		_init_eeaddr _nv_flags_ee		;--- save non volitle sys_flags
		mov	temp,nv_flags
		rcall	eewr
	
disp_nv_flag:	rcall	crlf				; formatting

		movw	zh:zl,xh:xl
		rcall	flash_str2con

		rcall	send_eq
		
		and	yl,nv_flags
		brne	disp_on			

disp_off:	_send_flash _off			; 
		rjmp	prompt	
disp_on:	_send_flash _on	
		rjmp	prompt


;-----------------------------------------------------------------------------
;---- get my call sign and save in eeprom
get_mycall:	rcall	arg_count
		cpi	temp,02
		brne	disp_mycall
	
		_init_eeaddr _mycallsign		; point to ee storage
		ldi	temp,2				; what arg to parse
		rcall	arg2call_ee			; save it

		rcall	eewr_zero_p			; zero termination					

disp_mycall:	rcall	crlf

		_send_flash _mycall
		rcall	send_eq

		rcall	init_dest_unp			; load all our ram buffers
		_init_z	_ax25_from_call
		rcall	disp_call

get_mycall_end:	rjmp	prompt


;----------------------------------------------------------------------------------
;---- get location dest and location path from console and store in eeprom
get_lpath:	ldi	xh,high(_lpath*2)
		ldi	xl,low(_lpath*2)
		_init_y	_loc_call	
		rjmp	get_call_path

;--- get location dest and location path from console and store in eeprom
get_unproto:  	ldi	xh,high(_unproto*2)
		ldi	xl,low(_unproto*2)
		_init_y	_unp_call
		
;--- get destination and store in ee (also handles any digis)
;--- x = msg text pointer, y = eeprom address
get_call_path:	rcall	arg_count
		cpi	temp,01
		breq	get_digi_disp		; just display

		cpi	temp,(max_digis + 4)	; test for too many arguments	
		brlo	pc + 2
		rjmp	cmd_err
			
		movw	eeaddr_h:eeaddr_l,yh:yl	; point to eeprom storage

		ldi	temp,2			; what arg to parse
		rcall	arg2call_ee		; save it

		rcall	eewr_zero_p		; zero termination, point to digi storage		

get_digis:	ldi	temp,3			; point to 3rd argument
		rcall	set_z2arg
		rcall	to_upper
		
		cpi	temp,'V'		; check for via 
		brne	get_digi_end		; no digis	

		rcall	arg_count
		
		mov	temp3,temp
		inc	temp3			; one extra for loop	

		ldi	ii,4			; point to forth argument
	
get_digi_lp:	mov	temp,ii			; get arg cnt
		rcall	arg2call_ee		; save it
		inc	ii
		cp	ii,temp3
		brne	get_digi_lp	

get_digi_end:	rcall	eewr_zero_p		; zero termination, or mark no digis	

get_digi_disp:	rcall	crlf
		movw	zh:zl,xh:xl
		rcall	flash_str2con
		rcall	send_eq

		rcall	init_ax25_dest		; copy eeprom values to ram buffers (so we can use display routines)		

		_init_z	_ax25_dest_call		; point to dest
		rcall	disp_call		; display it

disp_digi:	_init_z	_ax25_digi_cnt	

		ld	temp,z+
		tst	temp
		breq	get_unp_end		; if no digis just exit
	
		mov	ii,temp			; save length
		_send_flash _via		; console message	

		_init_z	_ax25_digi_buffer	; point to where shifted digis reside

disp_d_lp:	rcall	disp_call		; put on screen in callsign format
		subi	ii,7	
		breq	get_unp_end		; yep no more stations
		_send	','			; delim	
		rjmp	disp_d_lp		; go for next

get_unp_end:	rjmp	prompt			; done back to top	


;--- save the console arg from ram (_work_buff) to eeprom as callsign and ssid	
; eear  must be set, temp has arg number
arg2call_ee:	rcall	set_z2arg		; point z @ argh
		push	zl			; and save it
		push	zh

		ldi	temp2,6			; max 6 chars in call (plus 1 for ssid)

get_mc_lp:	ld	temp,z+			; get char
		tst	temp

		breq	get_mc_pad		; test for end or recognized delimiters

		cpi	temp,'-'
		breq	get_mc_pad

		cpi	temp,' '
		breq	get_mc_pad

		cpi	temp,','
		breq	get_mc_pad

		rcall	to_upper
		rcall	eewr_p			; write it

		dec	temp2			; six chars yet?
		breq	wr_ssid			; all done
		rjmp	get_mc_lp

get_mc_pad:	ldi	temp,' '		; fill unused locations with space char
		rcall	eewr_p
		dec	temp2
		brne	get_mc_pad

wr_ssid:	pop	zh			; get arg pointer again
		pop	zl

		ldi	temp2,7			; seven chars max

get_ssid:	subi	temp2,1
		brcs	bad_ssid

		ld 	temp,z+			; now, scan through buffer looking for '-'

		tst	temp			; test for end of string
		breq	bad_ssid	
	 
		cpi	temp,'-'
		brne	get_ssid	
		

grab_ssid:	rcall	asc2bin			; convert ascii str to int
		andi	zl,0x0f			; mask off top bits
		ori	zl,0x30			; or in top bits
		mov	temp,zl
		skip

bad_ssid:	ldi	temp,0x30		; something wrong with ssid, use default
		rcall	eewr_p			; write it to eeprom
		ret

;---------------------------------------------------------------------
;--- display some error messages
over_run_error:	cbr	ax25_flags,1<<over_run	; clear the error bit
		ldi	temp,01
		rcall   error_display
		rjmp	prompt			; to top	

error_display:	push 	temp	
		_send_flash _error
		_send	' '
		pop	temp
		rcall	send_byte
		ret

;---------------------------------------------------------------------
;--- copy raw ax25 buffer to vars and buffers
;--- trash z,y,temp
copy_packet2axbuffers:
get_s_d:	sts	_ax25_digi_cnt,zero	; default to zero digis

		_init_z	_ax25_rx_buffer		; point to beginning of rx buffer
		_init_y	_ax25_dest_call		; point to storage

		ldi	temp,14
		rcall	ram2ram			; z2y copy dest and source call to  ax25 buffer

		lds	temp,_ax25_rx_buffer+14	; get ssid of src call sign, leave z alone
		
		sbrs	temp,0			; see if low bit of from call ssid is set    
		rcall	save_digis		; yes, save digis in digi buffer 	

get_ctrl:	ld	temp,z+			; z should now be pointing at ctrl byte
		sts	_ax25_ctrl,temp		; save control
						; 
get_pid:	ld	temp,z+			; get pid
		sts	_ax25_pid,temp		; save it	

get_data:	_init_y	_ax25_rx_buffer		; z is pointing to start of data, y to storage

		lds	temp,_ax25_rx_cnt	; get total packet length

		add	yl,temp
		adc	yh,zero			; point to last char in message		

		sub	yl,zl			; get length of payload
		sbc	yh,zh
 
		cpi	yl,max_packet_data	; be sure it will fit in destination		
		brlo	get_data_a
		ldi	yl, max_packet_data	; hard limit data length	

get_data_a:	mov	temp,yl			; save it in temp, y is about to get clobbered
		_init_y	_ax25_rxtx_cnt		; point to data storage		
		st	y+,temp			; save total number of chars in payload
		rcall	ram2ram			; save payload (copy temp chars from z to y)
	
copy_p2w_end:	cbr	ax25_flags,1<<packet_rx ; allow for next packet
		ret

;--- save the digis in digi buffer (and char cnt first)
save_digis:	clr	temp2			; char counter (chars in digi path)
		_init_y _ax25_digi_buffer	; point to storage	

save_digi_lp:	inc	temp2			; bump char counter
		ld	temp,z+			; get char
		st	y+,temp
		sbrc 	temp,0			; if lsb is set we're done
		rjmp	save_digi_end		

		cpi	temp2,max_digis*7
		brne	save_digi_lp		; overflow


save_digi_of:	ld	temp,-y	 		; get last char stored
		sbr	temp,1<<0		; make it the last digi	
		st	y+,temp

		ld	temp,z+			; inc z to point past remaining digis to control byte
		sbrs	temp,0
		rjmp	pc - 2


save_digi_end:	sts	_ax25_digi_cnt,temp2	; save total chars at beginning of storage	
		ret


;--- display packet from ax_buffer
process_rcvd_packet:
		rcall	copy_packet2axbuffers	; copy packet from rx buffer to workspace 	
		sbrc	sys_flags,monitor	; see if monitor is enabled	
		rcall	display_ax_packet
		led1_off
		ret				; 

;--- ship formatted packet data to console	 
display_ax_packet:
		sbrc	sys_flags,trace		; if in trace mode, do a hex dump first
		rcall	packet_dump
	
;--- comment out next two test routines to try to display all packets (not just ax25 ui packets)
ctrl_test:	lds	temp,_ax25_ctrl
		cpi	temp,control_byte	; compare control
		breq	pc + 2			; match display	
		ret

pid_test:	lds	temp,_ax25_pid
		cpi	temp,pid_byte		; compare pid
		breq	pc + 2			; match display	
		ret
		
disp_from:	_init_z	_ax25_from_call
		rcall	disp_call
		_send	'>'

disp_to:	_init_z	_ax25_dest_call
		rcall	disp_call		
				
disp_digis:	lds	temp,_ax25_digi_cnt	; get count of chars in digi
		tst	temp
		breq	disp_data		; no digis
		_init_z	_ax25_digi_buffer	; point to digi storage
	
disp_digi_lp:	_send	','
		rcall	disp_call		; returns ssid in temp
		sbrc	temp,0			; if low bit is set, end of path		
		rjmp	last_digi		; disp call returns lsb of ssid in carry

		sbrs	temp,7			; was h bit set	
		rjmp	disp_digi_lp		; nope

		ldd	temp,z+6		; get ssid of next digi
		sbrc	temp,7
		rjmp	disp_digi_lp

		_send	'*'			; heard!
		rjmp	disp_digi_lp	

	
last_digi:	sbrs	temp,7			; last digi, if hi (h) bit set display * heard
		rjmp	disp_data
		_send	'*'

disp_data:	_send	':'		
		
		sbrc	nv_flags,mcom		; if mcom bit flag is set, display packet type 	
		rcall	disp_packet_type

		sbrc	nv_flags,header		; if header bit flag is set, put our packet data on a new line	
		rcall	crlf

		_init_z _ax25_rxtx_cnt		; point to storage
		ld	temp2,z+		; get payload length

		clr	temp			; if one char length string, keep our crlf routine happy
disp_data_lp:	mov	temp3,temp		; save next to last char
		ld	temp,z+
		rcall	putc
		dec	temp2
		brne	disp_data_lp

		cpi	temp,lf			; test last char
		brne	disp_rx_cr
		
		cpi	temp3,cr
		breq	disp_rx__end	

disp_rx_cr:	cpi	temp,cr
		breq	disp_rx_lf
		_send	cr

disp_rx_lf:	sbrs	nv_flags,lf_en
		ret
		_send	lf
disp_rx__end:	ret

;------------------------------------------------------------------------------
;--- display type of packet type on console  (pretty much just to keep ui-view happy)
disp_packet_type:
		_send	' '
		_send	'<'			; display formatting
		lds	temp,_ax25_ctrl		; get control byte	
		andi	temp,0x03		; mask off high 6 bits

disp_ui:	cpi	temp,0x03		; if both low bits are set, it's a ui frame	
		brne	disp_s
		_send	'U'
		_send	'I'			
		rjmp	disp_ptype_end
disp_s:						; add later if needed
disp_i:						; add later if needed
disp_ptype_err:	_send	'?'			; unknown type (for now)
disp_ptype_end:	_send	'>'
		_send	':'
		ret	

;------------------------------------------------------------------------------
;--- hex dump of packet buffer
packet_dump:	rcall	crlf
		_init_z	_ax25_dest_call
		rcall	ram7con			; dest	
		rcall	ram7con			; source

		_init_z	_ax25_digi_cnt
		ld	temp,z+
		tst	temp			; any digis?
		breq	packet_d_a
		
packet_d_lp:	rcall	ram7con			; display each digi
		sbrs	temp,0
		rjmp	packet_d_lp	
		_send	' '

packet_d_a:	lds	temp,_ax25_ctrl		; control byte	
		rcall	send_byte
		lds	temp,_ax25_pid		; pid byte
		rcall	send_byte
		_send	' '
		
		_init_z	_ax25_rxtx_buffer	; packet data as one long hex dump	
		lds	temp2,_ax25_rxtx_cnt
		rcall	ram2con_hex
		rcall	crlf
		ret

;-- send 7 chars in hex format to console followed by space
ram7con:	ldi	temp2,7	
		rcall	ram2con_hex
		push	temp
		_send	' '			; formatting
		pop	temp
		ret

;------------------------------------------------------------------------------
;--- display call and ssid on console
;--- z points to data, display 6 shifted chars, then ssid,
;--- return with ssid byte in temp 
;--- trash z and temp
disp_call:	push	ii			; need a counter
		ldi	ii,6			; num of chars
dp_loop:	ld	temp,z+			
		lsr	temp			; 
		cpi	temp,' '		; is space?
		breq	pc + 2			; don't ship	
		rcall	putc
		dec	ii
		brne	dp_loop

		ld	temp,z
		mov	ii,temp			; get ssid				
		lsr	ii			; shift off lsb	
		andi	ii,0x0f			; mask off top nibble
		breq	ssid_end		; if  ssid = zero display nothing	

		_send	'-'

		cpi	ii,10			; < 10 ? 
		brlo	ssid_1_9			

		_send	'1'			; > 10 ?
		subi	ii,10			; adjust

ssid_1_9:	mov	temp,ii
		rcall	send_nibble		; display ssid
	
ssid_end:	ld	temp,z+			; return with ssid
		pop	ii	
		ret

;------------------------------------------------------------------------------
;--- ax25 send formated data routines
;------------------------------------------------------------------------------
;--- send strings from rs-232 buffer to eeprom stored unproto destination
ax_send_str:    rcall	wait_dcd		; wait for clear channel
		rcall	init_dest_unp		; send to unproto dest and path
		rcall	ptt_key			; key radio, send initial flags

ax_send_st_lp:	rcall	get232_str2axbuff	; copy data from rs232 circ buffer			
		sts	_ax25_rxtx_cnt,temp	; copy temp returned length		
		rcall	ax_crlf			; terminate it		
		rcall	ax_send_packet
		rcall	display_ax_packet	; display it	

		sbrc	sys_flags,ctrl_c_rcvd	; did we get a contol-c?
		rjmp	ax_send_str_end		; yes
		tst	cr_cnt			; any more strings?	
		brne	ax_send_st_lp		;   yes, get another		
ax_send_str_end:
		rcall	ptt_unkey
		ret

;------------------------------------------------------------------------
;--- wait for free channel, send beacon text to eeprom path
;--- trash z,y 
ax_send_beacon:	rcall	wait_dcd			; wait for clear channel

		sts	_ax25_rxtx_cnt,zero		; null out our buffer		
		_init_eeaddr _btext_ee			; point to beacon text

ax_btext_lp:	rcall	get_ee_ascii	
		breq	ax_btext_cr			; all done
		rcall	add2axbuff
		rjmp	ax_btext_lp	

ax_btext_cr:	rcall	ax_crlf				; terminate tx data string

		rcall	init_dest_unp			; copy default destination	
		_init_y _ax25_dest_call			; we will overwrite destination as beacon
		_init_zflash	_beacon
		

b_dest_lp:	lpm	temp,z+				; get BEACON from flash
		lsl	temp				; be sure flash stored string is 6 chars followed by zero
		st	y+,temp
		brne	b_dest_lp			; lsl tested for zero

		ldi	temp,0x60			; ssid 0 left shifted
		st	-y,temp	


b_dest_a:	sts	_btimer,zero			; zero beacon timer
		sts	_btimer+1,zero
		rjmp	send_it				; send and display
		
;------------------------------------------------------------------------
;--- load ax buffer with raw string in wx uart buffer and send to ee destination, append crlf
;--- trash z,y
ax_send_wx:	rcall	wait_dcd		; wait for clear channel

		_init_z	_wx_rx_cnt		; point to wx string storage	
		_init_y	_ax25_rxtx_cnt		; point to ax25 tx buffer 

		ld	temp,z+			; get char count
		tst	temp
		breq	send_wx_end		; if no data, do nothing

		cpi	temp,max_packet_data	; see if it will fit
		brlo	pc + 2
		ldi	temp,(max_packet_data-2); nope copy what we can, leave room for crlf
		st	y+,temp			; save it
		rcall	ram2ram			; copy temp chars from z to y 
		rcall	ax_crlf			; add termination
		
		rcall	flush_wx_buff		; clear buffer and wx cr bit flag		
	
		rcall	init_dest_unp		; sending to unproto dest and path
send_it:	rcall	ax25_send		; send one packet
		rcall	display_ax_packet	; display it
send_wx_end:	ret


;------------------------------------------------------------------------
;--- load ax buffer with formatted position, and send to ee destination
;--- tracking string must be in rmc buffer and valid
;--- depending on fixtype either send rmc string or aprs type fix with optional comment.
ax_send_posit:	sbrs	tracking_flags,rmc_rcvd	; do we have a good rmc string to parse data? 
		rjmp	ax_posit_err

		sbrs	tracking_flags,tracking	; are we tracking?
		rjmp	ax_posit_err

		rcall	wait_dcd		; wait for clear channel

		_init_eeaddr _fix_type_ee	; get fix type
		rcall	eerd
		tst	temp	
		breq	send_aprs		; zero is aprs, all else is rmc for now
						; maybe add mic-e in the future
send_rmc:	_init_z	_rmc_cnt		; point to rmc data storage	
		_init_y	_ax25_rxtx_cnt		; point to ax25 tx buffer 

		ld	temp,z+			; get char count
		cpi	temp,max_packet_data	; see if it will fit
		brlo	pc + 2
		ldi	temp,(max_packet_data-2); nope, copy what we can, leave room for crlf
		st	y+,temp			; save it
		rcall	ram2ram			; copy temp chars from z to y 
		rjmp	pack_cr			; add cr and send the data
	
send_aprs:	sts	_ax25_rxtx_cnt,zero	; null out our buffer		
		ldi	temp,'!'
		rcall	add2axbuff

pack_lat:	ldi	ii,3			; point to 3rd argument (lat)
		rcall	z_2_gps_arg

		ldi	ii,4
		rcall	ram2axbuff

pack_lat_dp:	ld	temp,z			; get decimal point
		cpi	temp,'.'
		brne	ax_posit_err		; some primitive error checking here
		rcall	add2axbuff			
	
pack_lat_last:	ldd	temp,z+1
		rcall	add2axbuff
		ldd	temp,z+2
		rcall	add2axbuff
		
		rcall	next_gps_arg

		ld	temp,z			; get north south
		rcall	add2axbuff

pack_pos_sep:	_init_eeaddr _pos_table_char	;/ sybmol table
		rcall	eerd
		rcall	add2axbuff		


pack_lon:	rcall	next_gps_arg	
		ldi	ii,5
		rcall	ram2axbuff


pack_lon_dp:	ld	temp,z			; get decimal point
		cpi	temp,'.'
		brne	ax_posit_err		; some primitive error checking here
		rcall	add2axbuff			
	
pack_lon_last:	ldd	temp,z+1
		rcall	add2axbuff
		ldd	temp,z+2
		rcall	add2axbuff
		
		rcall	next_gps_arg			; get east west
		ld	temp,z
		rcall	add2axbuff

		_init_eeaddr _pos_sym_char
		rcall	eerd				; get symbol
		rcall	add2axbuff		
	
		rcall	next_gps_arg			
		rcall	next_gps_arg			; point past speed
		rcall	copy_formated_num
		
		ldi	temp,'/'
		rcall	add2axbuff

		ldi	ii,7
		rcall	z_2_gps_arg
		rcall	copy_formated_num
		
		_init_eeaddr	_ltext_ee		; add location comment to output buffer
pack_comment_lp:
		rcall	get_ee_ascii			; get ascii char with translation support from eeprom
		breq	pack_cr				; at end of string?

		rcall	add2axbuff			; add to buffer
		rjmp	pack_comment_lp	


pack_cr:	rcall	ax_crlf				; terminate data
send_2_lpath:	_init_y	_loc_call			; eeprom pointer to lpath dest data
		rcall	init_ax25_dest			; get lpath dest data from eeprom into ax25 tx buffers
		rcall	send_it				; send it out

ax_posit_err:
		sts	_ltimer,zero			; zero location timer
		sts	_ltimer+1,zero	
		ret

;------------------------------------------------------------------------
;--- add a cr and optional lf to ax25 tx buffer
ax_crlf:	ldi	temp,cr
		rcall	add2axbuff	
		sbrs	nv_flags,ax_lf_en
		ret
		ldi	temp,lf
		rcall	add2axbuff
		ret

;--- reload 16 bit ram timer from eeprom
;--- eeaddr must be set and z pointing to ram location
reload_timer_16:
		rcall	eerd_p
		cli						; reload timer
		std	z+0,temp
		rcall	eerd
		std	z+1,temp
		reti

;--- NMEA parsing related routines 
;--- set z to arg ii within rmc buffer
z_2_gps_arg:	_init_z	_rmc_buffer		;	
z_2_gps_arg_lp:	tst	ii
		brne	pc+2
		ret
		rcall	next_gps_arg
		dec	ii
		rjmp	z_2_gps_arg_lp	

;--- discard chars til a '.'
next_gps_arg:	ld	temp,z+
		cpi	temp,','
		brne	next_gps_arg	
		ret

;--- copy var length track/speed data to fixed field
copy_formated_num:
num_len:	ldi	temp,0xff		
		movw	yh:yl,zh:zl		; get copy of pointer

num_len_lp:	inc	temp			;--- check length of num string@z 
		ld	temp2,y+
		cpi	temp2,'0'
		brlo	copy_form_a
		cpi	temp2,'9'+1		
		brlo	num_len_lp	
		
copy_form_a:	ldi	ii,3
		sub	ii,temp
		push	temp			; save char count

copy_form_lp:	tst	ii			; stuff ii zeros to our ax25 buffer
		breq	copy_form_c	

		ldi	temp,'0'
		rcall	add2axbuff
		dec	ii
		rjmp	copy_form_lp		

copy_form_c:	pop	ii			; fall into ram copy routine
	
;--- copy ii chars from ram to axbuffer
ram2axbuff:	tst	ii
		brne	pc + 2
		ret
		ld	temp,z+			; get char
		rcall	add2axbuff		; save it
		dec	ii			; dec char counter
		rjmp	ram2axbuff		; go for next

;--- add char to end of linear buffer, inc size/cnt, don't exceed buffer size
; trash y
add2axbuff:	_init_y	_ax25_rxtx_cnt
		ld	temp2,y				; get current count
		cpi	temp2,max_packet_data
		breq	addaxbuff_end			; no room, toss char

		inc	temp2				; bump and save
		st	y,temp2

		add	yl,temp2			; adjust
		adc	yh,zero
		st	y+,temp				; store data
addaxbuff_end:	ret


;------------------------------------------------------------------------------
;--- send packet from ram buffer
;--- fill all buffers first, channel should be clear! no dcd testing here! 
ax25_send:	rcall	ptt_key
		rcall	ax_send_packet

;--- wait for last flag byte, unkey radio
ptt_unkey:	rcall 	ax25_tx_wait			; wait for last flag to be sent	
		ldi	ii,tx_hang			
		rcall	del_hunds
		unkey_radio				;  release ptt

		_rx_mx614				;  back to rx mode
		cbi	MX_PORT,MX_TXD			;  clear data out line, if high mx614 sends backchannel tone
		sbr	ax25_flags,(1 << rx_enable)  	;  re-enable isr reception
		ret

;--- change mode, key radio, send initial flags
ptt_key:	cbr	ax25_flags,1<<rx_enable 	; turn off isr reception , probably does not matter
		ldi	temp,control_byte		; set default ax25 control and pid
		sts	_ax25_ctrl,temp	
		ldi	temp,pid_byte			; these could be hardcoded, but we'll do it this way for future flexibility
		sts	_ax25_pid,temp

		_tx_mx614				; change mode on modem chip	
		key_radio				; ptt down

		
		_init_eeaddr _tx_delay_ee			; get tx delay value
		rcall	eerd		
		rcall	send_ax_flags			; start packet flag bytes

		ret

;--- send a packet, no radio control 
ax_send_packet:	ldi	temp,1		
		rcall	send_ax_flags			; start packet flag bytes
			
		ldi	temp,0xff			; init crc
		sts	_tx_crc_lo,temp	
		sts	_tx_crc_hi,temp	

		rcall	ax25_tx_wait			; wait till flag sent to re-enable bitstuffing
		sbr	ax25_flags,(1<<tx_bit_stuff)	; turn on bit stuffing

send_to:	_init_z	_ax25_dest_call			; point	to destination
		ldi	temp,7
		rcall	send_ax				; send destination

send_from:	ldi	temp,7
		rcall	send_ax				; send mycall	
 
		sbrc	temp,0				; temp should be last byte sent
		rjmp	send_ctrl			; if low bit of mycall ssid was set, no digis		

send_digi:	lds	temp,_ax25_digi_cnt		; get length of digi path
		_init_z _ax25_digi_buffer		; point to digi path
							; lsb of final ssid must already be set!
		rcall	send_ax				; send digis

send_ctrl:	lds	temp,_ax25_ctrl			; control byte
		rcall	putc_ax

send_pid:	lds	temp,_ax25_pid			; pid byte
		rcall	putc_ax

send_data:	_init_z	_ax25_rxtx_cnt			; send the packet body

		ld	temp,z+				; get count, point to data
		rcall	send_ax				; send it

		rcall	send_ax_crc			; send crc

		ldi     temp,tail_flags			; send final flag byte
		rcall	send_ax_flags			
		ret

;******* ax25 send data sub routines *******************************************
;--- send temp number of bytes from z to radio
; set z before calling, trashes z, temp
send_ax:	push	ii
		mov	ii,temp

send_ax_lp:	tst	ii
		breq	send_ax_end
		ld	temp,z+		; get char
		rcall	putc_ax		; put it out
		dec	ii
		rjmp	send_ax_lp

send_ax_end:	pop	ii
		ret	

;--- send temp number of flag bytes
send_ax_flags:	push	ii
		mov	ii,temp
sendf_lp:	rcall	ax25_tx_wait					; make sure buffer is empty	
		cbr	ax25_flags,(1<<tx_bit_stuff)			; turn off bit stuffing and crc gen
		ldi	temp,0x7e		
		rcall	putc_ax
		dec	ii
		brne	sendf_lp
		pop	ii
		ret

;--- send calculated crc, lo byte first
send_ax_crc:	lds	temp,_tx_crc_hi		; get high first, sending low will change crc
		push	temp
	
		lds	temp,_tx_crc_lo			; get it	
		rcall	putc_ax_comp			; send it

		pop	temp				; get high
		;rcall	putc_ax_comp			; send it out
		;ret

;--- wait for empty buffer, then load buffer and set bit count, crc the byte after sending
putc_ax_comp:	com	temp						; stubb to send complimented data (crc)	
putc_ax:	rcall	ax25_tx_wait
		mov	ax25_txbuff,temp

		ldi	temp2,1<<tx_cnt3
		or	ax25_tx_counts,temp2				; 8 bits to send

		rcall	ax25_tx_crc					; generate crc on data we just sent	
		sbrc	sys_flags,debug					; if debugging, hex dump		
		rcall	send_byte
		ret

;--- wait for buffer to empty/last bit was sent
ax25_tx_wait:	push	temp
ax25_txw_lp:	wdr
		mov	temp,ax25_tx_counts
		andi	temp,0b00001111
		brne	ax25_txw_lp
		pop	temp
		ret	

;------------------------------------------------------------------------------
; generate crc for temp data, init the crc buffer first
; trash x,y
ax25_tx_crc:	push	ii
		lds	yh,_tx_crc_hi			; get crc
		lds	yl,_tx_crc_lo
				
		ldi 	xh,0x84				; preload with 16 bit poly for speed 	
		ldi 	xl,0x08 			; 
		eor 	yl,temp 			; xor in data

		ldi	ii,8				; 8 bits

crc_tx_lp:	lsr 	yh
		ror 	yl
		brcc 	crc_tx_1
		eor 	yl,xl
		eor 	yh,xh
crc_tx_1:	dec	ii
		brne	crc_tx_lp
			
		sts	_tx_crc_hi,yh			; save results
		sts	_tx_crc_lo,yl
		pop	ii
		ret


;--- wait for clear channel, display packets while waiting, handle misc housekeeping 
; uses persist / slot time (that is, if I got the algorithm right) 
wait_dcd:	wdr				; feed the watchdog timer	
		sbrc	ax25_flags,packet_rx
		rcall	process_rcvd_packet

		sbrc	tracking_flags,aux_cr_rcvd
		rcall	get_rmc

		sbis	INH_PORT,INH
		rjmp	wait_dcd		; test inhibit line, (will wait forever!)	

		sbis	MX_INP,DET		; check pin
		ret				; is low, just return
	
wait_dcd_lp:	wdr				; feed the watchdog timer	
		sbrc	ax25_flags,packet_rx
		rcall	process_rcvd_packet	

		sbrc	tracking_flags,aux_cr_rcvd
		rcall	get_rmc

		sbic 	MX_INP,DET		; was high wait for low	
		rjmp	wait_dcd_lp

		_init_eeaddr _slot_ee
		rcall	eerd_p			; get slot time	

wait_dcd_a:	lds	temp2,_ms_10		; get current count from ram, isr decs at 100hz
wait_dcd_b:	sbic	MX_INP,DET
		rjmp	wait_dcd_lp		; hit carrier, back to top	

		lds	temp3,_ms_10		
		cp	temp2,temp3
		breq	wait_dcd_b	
		dec	temp
		brne	wait_dcd_a		; big loop

		rcall	get_random		; get random
		mov	temp2,temp

		rcall	eerd			; get persist value
		sub	temp,temp2		; see if random is eq to or lower then persist	
						; exit, else loop again
		breq	wait_dcd_end		; rand = persist
		brlo	wait_dcd_lp		; rand > persist, higher try again

wait_dcd_end:	ret
			

;------------------------------------------------------------------------------
;--- get a pseudo random 8 bit value via linear congruential algorithm 
;--- we may want to beef up the random number generator in the future
get_random:	lds	temp,_random			; get last value from ram
		ldi	temp2,0x1d			; save in both temp2 and temp
            	clc
            	rol     temp
            	brcc	pc + 2
            	eor   	temp,temp2
            	sts	_random,temp
		ret	

;------------------------------------------------------------------------------
;--- copy eeprom stored mycall,dest and digipath to ax_tx_buffer 
;--- y=eeprom stored dest
init_dest_unp:	_init_y	_unp_call
init_ax25_dest:	
		_init_z	_ax25_from_call
		_init_eeaddr _mycallsign
		ldi	temp3,7			; max chars to copy (6 for station + ssid)
		rcall	ee2ram_shifted

		movw	eeaddr_h:eeaddr_l,yh:yl
		_init_z _ax25_dest_call
		rcall	ee2ram_shifted		; temp3 should still be 7

		rcall	inc_eeaddr		; bump to point digis 
		_init_z	_ax25_digi_buffer

		ldi	temp3,max_digis*7
		rcall	ee2ram_shifted		; move the data, temp returns length

		sts	_ax25_digi_cnt,temp	; save total chars at beginning of storage
		tst	temp			; test digi length
		brne	init_ax_1	

		lds	temp,_ax25_from_call+6	; if no digis, set low bit of mycall ssid to indicate end of path 
		sbr	temp,1<<0
		sts	_ax25_from_call+6,temp	
		ret

init_ax_1:	ld	temp,-z	 		; get last char stored
		sbr	temp,1<<0		; set the lsb to indicate end of digi path	
		st	z+,temp

init_ax25_end:	ret		

;----------------------------------------------------
;---- delay ii * 10 ms (approx) 
del_hunds:	lds	temp2,_ms_10					; get current count
del_h_loop:	wdr
		lds	temp,_ms_10		
		cp	temp2,temp
		breq	del_h_loop	
		dec	ii
		brne	del_hunds
		ret

;------------------------------------------------------------------------------
;---  data extraction routines
;------------------------------------------------------------------------------
;--- test arg pointed @temp, set carry if 1 or second char is 'n' 
on_off:		rcall	set_z2arg		; point z @ argh
		ld	temp,z+
		cpi	temp,'0'		; 0 is off
		breq	arg_is_off
		cpi	temp,'1'		; is on
		breq	arg_is_on	

		ld	temp,z			; get second char of argument
		rcall	to_upper
		cpi	temp,'N'		; is on (maybe)
		breq	arg_is_on		; all else is off		
		
arg_is_off:	clc
		skip
arg_is_on:	sec
		ret


;*********** ascii data parsing routines ******************** 
;--- most assume use of the z register, and the _work_buffer space
;--- buffer should be zero terminated, 
;--- position z at first arg on cmd line, 
first_arg:	_init_z	_work_buff
		rcall	discard_white		; toss white chars 
		ret

;--- set z to first char of arg temp, returns that char in temp
set_z2arg:	push	ii
		mov	ii,temp
		rcall	first_arg		; point to beginning of first arg

set_z_lp:	dec	ii
		breq	set_z_end
		rcall	next_arg	
		rjmp	set_z_lp
		
set_z_end:	pop	ii
		ret

;--- calc len of argument pointed to by z
arg_len:	push	zl
		push	zh
		rcall	discard_nw
		pop	zh
		pop	zl
		ret

;--- count total number of parameters in string, parameters are seperated by ' ' ','
;--- return temp
arg_count: 	push	ii
		ldi	ii,255
		rcall	first_arg				; point to beginning of data	

count_arg_lp:	inc	ii
		rcall	next_arg				; if next arg returned a null at end of str
		ld	temp,z
		brne	count_arg_lp

		mov	temp,ii
		pop	ii
		ret
	
;--- inc z to discard spaces,cr,lf, and delim chars (' '',')
discard_white:	ld	temp,z+
		cpi	temp,' '
		breq	discard_white
		cpi	temp,','
		breq	discard_white
		cpi	temp,cr
		breq	discard_white
		cpi	temp,lf
		breq	discard_white

		sbiw	zl,1					; undo last z+ 
		ret
		
;--- inc z till a ' ',',' or end of string,			
;--- returns temp = num of chars, max 255 
discard_nw:	push	ii
		ldi	ii,255

disc_nw_lp:	inc	ii
		ld	temp,z+

		tst	temp					; test for zero termination
		breq	disc_nw_end

		cpi	temp,','
		breq	disc_nw_end	

		cpi	temp,' '
		brne	disc_nw_lp	

disc_nw_end:	sbiw	zl,1					; adjust for that last z+
		mov	temp,ii					; return count
		pop	ii
		ret

;--- position zl at first char of next argument, must start on valid char of current arg
next_arg:	ld	temp,z
		tst	temp
		breq	next_arg_end				; test for end of string
		rcall	discard_nw				; discard current arg
		rcall	discard_white				; discard anything after current argh
next_arg_end:	ret



;------------------------------------------------------------------------------
;-------------------- base conversion routines --------------------------------
;------------------------------------------------------------------------------
;--- convert temp to upper case
to_upper:	cpi 	temp,'a'
		brlo	toupper_end
		cpi 	temp,'z'+1
		brsh	toupper_end
		subi	temp,' '
toupper_end:	ret		

;--- convert byte to two ascii chars and store @ z, hi byte first
hex2ram:	push	temp
		swap	temp
		rcall	lo_hex2ram
		pop	temp
		
lo_hex2ram:	rcall	hex_asc
		st	z+,temp
		ret

;--- routine converts 0-f hex in temp to 0 to F ascii
hex_asc:	andi   	temp,0b00001111     	;lower nybble only
   		cpi   	temp,10  		;0-9 or A-F?
   		brlo	hex_asc_1           	;
   		subi	temp,-7    		;adjust A-F
hex_asc_1:	subi   	temp,-48  		;add offset to convert to ascii
		ret

;--- convert asci char in temp to 0-15 nibble
;--- sets carry if char is invalid (not 0-9 or a-f)
asc2hex:	rcall	to_upper		; convert to upper case 
		cpi	temp,'F'+1		; test for 'E' and above
		brsh	asc_error
		cpi	temp,'A'
		brsh	asc_letters
		cpi	temp,'9'+1
		brsh	asc_error
		cpi	temp,'0'
		brsh	asc_numbers

asc_error:	clr	temp			; return zero and error
		sec				; set carry to indicate error
		ret				; and out

asc_letters:	subi	temp,0x07		;A-F ('0'+7) is the same as ('A' - 10)
asc_numbers:	subi	temp,'0'		;0-9	
		clc				; indicate sucess
		ret				; and out

;------------------------------------------------------------------------------
;--- send un-signed 16 bit bin to console as ascii, leading zero supression:
;--- enter with binary in x
;--- trash z,x,temp, and uses t bit for zero suppression
bin2console:	clr	temp
		or	temp,xh
		or	temp,xl			; test for zero
		breq	b2c_end			; if zero, just print a single '0'

bin2c1:		clt				; t bit used for leading zero suppression, set first time non zero digit is printed

		ldi	zh,high(10000)		; how many tens of thousands
		ldi	zl,low(10000)
		rcall	sub_ascii

		ldi	zh,high(1000)		; how many thousands	
		ldi	zl,low(1000)
		rcall	sub_ascii
		
		ldi	zh,high(100)		; how many hundreds
		ldi	zl,low(100)
		rcall	sub_ascii

		ldi	zl,low(10)		; tens
		rcall	sub_ascii

b2c_end:	mov	temp,xl			; ones
		subi	temp,-'0'		; just convert to ascii
		rjmp	sub_asc_b		; send it and exit	

sub_ascii:	ldi	temp,'0'		; start at ascii zero
sub_asc_lp:	cp	xl,zl			; see if we can sub without a carry
		cpc	xh,zh
		brlo	sub_asc_a		; no, done with this loop	
		sub	xl,zl
		sbc	xh,zh			; actual subtract
		inc	temp			; bump loop counter (in ascii)
		rjmp	sub_asc_lp		; loop again

sub_asc_a:	brts	sub_asc_b		; leading zero supression support		
		cpi	temp,'0'		; 
		breq	sub_asc_end		; 
		set				; non zero digit sent to console					
sub_asc_b:	rcall	putc			; send it out
sub_asc_end:	ret		


;------------------------------------------------------------------------------
;--- convert ascii string to int (16 bits max)
;--- enter with z pointing to string, returns in z
asc2bin:	push	xl		; workspace
		push	xh
		clr	xl		; start at zero
		clr	xh

a2b_l:		ld	temp,z		; get char
		cpi	temp,'9'+1
		brsh	a2b_end
		cpi	temp,'0'
		brlo	a2b_end
		rcall	mul_x_10
		ld	temp,z+		; get it again

		subi	temp,'0'

		add	xl,temp		; add new digit
		adc	xh,zero
		rjmp	a2b_l		; get next digit

a2b_end:	mov	zl,xl
		mov	zh,xh

		pop	xh		; restore workspace
		pop	xl
		ret

;---------------------------------------------------------------------------
;--- multiply x register by 10 decimel
mul_x_10: 	push	xh		; save a copy
		push	xl		; 

		rcall	rol_x		; * 2
		rcall	rol_x		; * 4

		pop	temp
		add	xl,temp		; * 5
		pop	temp
		adc	xh,temp		; fall into final *2

;--- rotate x register left by one position (multiply by 2)		 
rol_x:		lsl	xl
		rol	xh
		ret

;------------------------------------------------------------------------------
;--- copy zero terminated string from z to ee:
;--- set z and eeaddr first, temp2 is maximum num of chars
ram_str2ee:	subi	temp2,1			; test for limit
		brcc	ram_str_ee_1
		clr	temp
		skip
ram_str_ee_1:	ld	temp,z+
		rcall	eewr_p
		tst	temp
		brne	ram_str2ee
		ret		

;--- copy temp length block from z to y
ram2ram:	tst	temp
		brne	pc + 2
		ret
		ld	temp2,z+
		st	y+,temp2
		dec	temp
		rjmp	ram2ram

;--- copy temp2 chars from flash @ z to eeprom:
;--- set z and eeaddr first
flash2ee:	tst	temp2
		brne	pc + 2
		ret
		lpm	temp,z+
		rcall	eewr_p
		dec	temp2
		rjmp	flash2ee

;------------------------------------------------------------------------------
;---------------- send data to console routines ------------------------------- 
;------------------------------------------------------------------------------
;--- send quote symbol to console
send_quote:	ldi	temp,'''		; done often enough to save code space as a sub call
		rjmp	putc
;--- send equals symbol to console		; done often enough to save code space as a sub call
send_eq:	ldi	temp,'='
		rjmp	putc

;--- send a cr and optional lf to console 
crlf:		_send	cr
		sbrs	nv_flags,lf_en
		ret
		_send	lf
crlf_end:	ret	
		
;--- send nibbles as ascii		
send_hi_nibble:	swap	temp
send_nibble:	rcall	hex_asc
		rcall	putc
		ret

;--- send byte to console as two asc chars
send_byte:	push	temp		; save temp
		rcall	send_hi_nibble	; send hi nibble
		pop	temp		; reget data	
		push	temp		; and save it again
		rcall	send_nibble
		pop	temp		; just restores temp
		ret

;--- copy codespace stored strings to console atmega specific
flash_str2con:	lpm	temp,z+		; get char from codespace
		tst	temp		; test for end	
		brne	pc + 2
		ret
		rcall	putc		; send it
		rjmp	flash_str2con

;--- send temp2 chars from z to console
ram2con:	tst	temp2
		brne	pc + 2
		ret	
		ld	temp,z+			; get char
		rcall	putc		
		dec	temp2
		rjmp	ram2con	

;--- send temp2 chars from z to console as hex
ram2con_hex:	ld	temp,z+			; get char
		rcall	send_byte		
		dec	temp2
		brne	ram2con_hex	
		ret

;--- copy strings from ee to console
ee2con:		rcall	eerd_p		;  get char from ee
		breq	ee2con_end 	;  test for zero terminator
		rcall	putc		;  output char 
		rjmp	ee2con		;  and loop for next
ee2con_end:	ret


;------------------------------------------------------------------------------
;--- test string from aux serial port, and if valid copy the string data to rmc buffer
;--- set flag bits on valid strings and valid tracking  
;--- trashes temp,temp2,temp3, z and y
get_rmc:	lds	temp3,_aux_rx_cnt		; how many chars in string
		cpi	temp,min_nmea_len		; test for minimum length string
		brsh	pc + 2	
		rjmp	get_rmc_end			

rmc_test:	_init_y	_aux_rx_buffer			; make sure our string starts with '$GPRMC'
		_init_zflash	_filter_string		; getting compare from flash
		
rmc_lp:		ld	temp,y+				; get ram char
		lpm	temp2,z+			; get flash char
		
		tst	temp2				; flash was zero terminated
		breq	rmc_chksum			; all done

		cp	temp,temp2			; actual compare of chars
		breq	rmc_lp     			; get next char
		rjmp	get_rmc_end			; failed , discard		
	
rmc_chksum:	ldi	temp2,gprmc_checksum		; storage for chksum, load with precalculated checksum of 'GPRMC'
		_init_z	_aux_rx_buffer+6		; point past $GPRMC  

chksum_lp:	dec	temp3					
		breq	get_rmc_end			; an out if string does not have a '*'

		ld	temp,z+				; get char
		cpi	temp,'*'
		breq	chksum_a
		eor	temp2,temp			; sum the chars between $GPRMC and * for chksum

		rjmp	chksum_lp			; next char

chksum_a:	ld	temp,z+	
		rcall	asc2hex
		swap	temp

		eor	temp2,temp			; mask out high nibble

		ld	temp,z+
		rcall	asc2hex
		eor	temp2,temp			; low nibble 
		brne	get_rmc_end			; failed checksum
		
rmc_copy_it:	_init_z _aux_rx_buffer			; source
		_init_y	_rmc_buffer			; destination
	
		lds	temp,_aux_rx_cnt		; how many chars	
		cpi	temp,rmc_max_len + 1		; check for over run (should never happen)
		brlo	pc + 2
		ldi	temp,rmc_max_len		; clip to maximum length

		sts	_rmc_cnt,temp			; store char cnt
		rcall	ram2ram				; copy it
			
get_rmc_b:	sbr	tracking_flags,1<<rmc_rcvd	; flag that a good (checksum at least) rmc string was received	
		sts	_rmc_exp_timer,zero							;  
		sts	_rmc_exp_timer+1,zero

		ldi	ii,2				; see if we are tracking (active)
		rcall	z_2_gps_arg

		ld	temp,z
		cpi	temp,'A'			; A means active
		brne	get_rmc_no_track	

		sbr	tracking_flags,1<<tracking
		led2_on					; status / indicator
		rjmp	get_rmc_end	

get_rmc_no_track:
		cbr	tracking_flags,1<<tracking
		led2_off

get_rmc_end:	rcall	flush_aux_buff			; make rooom for next string
		ret

;---------------------------------------------------------------------------
;--- get zero terminated string from eeprom and send out our aux port
gps_init:	_init_eeaddr	_gpsinit_text_ee		

gps_init_lp:	rcall	get_ee_ascii			; get a char
		breq	gps_init_end			; all done or no string
	
gps_init_a:	rcall	putc_aux			; send char and loop
		rjmp	gps_init_lp
gps_init_end:	ret


;--- get ascii byte from eeprom
;--- translates #xx into raw ascii code
get_ee_ascii:	rcall	eerd_p				; get char
		breq	get_ee_asc_end			; all done or no string

		cpi	temp,'#'						
		brne	get_ee_asc_end			; esc char
							; get three chars	
		rcall	eerd_p				; get next char
		breq	get_ee_asc_end			; error
	
		rcall	asc2hex				; convert ascii to hex nibble	

		mov	temp2,temp			; save and swap to high nibble
		swap	temp2

		rcall	eerd_p				; get another char
		breq	get_ee_asc_end			; error
		rcall	asc2hex
		or	temp,temp2
		clz					; so our calling routine does not get confused						
		
get_ee_asc_end: ret

;------------------------------------------------------------------------------
;--- copy zero terminated strings from ee to ram
;--- set eeaddr and z before calling. temp3 = max num of chars
;--- if shifting is enabled, shift byte first before writing , t bit used for shift support!
;--- don't copy the zero, return length in temp
ee2ram_shifted:	set			; stubb to enable shifting of data
		rjmp	pc + 2			
ee2ram:		clt
		clr	temp2
ee2ram_lp:	rcall	eerd_p
		breq	ee2ram_end

		brtc	pc + 2
		lsl	temp
		st	z+,temp
		inc	temp2

		cp	temp2,temp3	; test for max length
		breq	ee2ram_end		

		rjmp	ee2ram_lp
	
ee2ram_end:	clt	
		mov	temp,temp2
		ret

;------------------------------------------------------------------------------
;--- basic eeprom support, eeaddr_h eeaddr_l must be set
eewr_zero_p:	clr	temp		; stub to write zero at current ee address, will inc	
eewr_p:		rcall	eewr		; ee write w/inc of ee address
		skip
eerd_p:		rcall	eerd		; ee read w/inc of ee address also tests temp before return
inc_eeaddr:	inc	eeaddr_l
		brne	pc + 2
		inc	eeaddr_h	
		tst	temp
		ret	

;--- ee write temp
eewr:		sbic	EECR,EEWE	; if EEWE not clear
		rjmp	eewr		; keep waiting

		out	EEARl,eeaddr_l
		out	EEARH,eeaddr_h
		out	EEDR,temp	; output the data

		in	temp,sreg	; save current global int status
		cli			; turn off ints for write
		sbi 	EECR,EEMWE	; set master write enable	
		sbi	EECR,EEWE	; set EEPROM Write strobe
		sbrc	temp,7		; if ints were off, leave them off
		sei
		rjmp	ee_end

;--- ee read , returns in temp
eerd:		sbic	EECR,EEWE	; if EEWE not clear
		rjmp	eerd		; keep waiting

		out	EEARl,eeaddr_l
		out	EEARh,eeaddr_h
		sbi	EECR,EERE	; set EEPROM Read strobe

ee_end:		in	temp,EEDR	; get data	
		out	EEARh,zero	; clear ee address to prevent data corruption
		out	EEARl,zero		
		ret 

;------------------------------------------------------------------------------
;--- scan through cmd table on hit, copy command address into z register
;--- if no match, copy default address into z
get_cmd_addr:	rcall	first_arg	; z pojnt to first arg on line
		movw	yh:yl,zh:zl	; copy into y		

		ldi	zl,low(_cmd_table *2)	
		ldi	zh,high(_cmd_table *2)	

get_cmd_lp:	lpm	temp,z		; test for end of table
		tst	temp
		breq	get_cmd_fail	
	
		rcall	cmp_fz2ry
		brcs	get_cmd_sucess	; hit!
					; fail, inc y to point to next cmd string
get_cmd_disc:	lpm	temp,z+		; inc y to discard rest of string and terminating zero
		tst	temp
		brne	get_cmd_disc

		sbrc	zl,0		; point past table address	
		adiw	zl,1		; keep in mind byte boundries		
		adiw	zl,2		
		rjmp	get_cmd_lp	
			

get_cmd_sucess:	lpm	temp,z+		; get table stored address
		lpm	temp2,z+	

		mov	zh,temp
		mov	zl,temp2		
		ret			; all done with command address in z

get_cmd_fail:				; no match, load default address	
		ldi	zh,high(cmd_err)
		ldi	zl,low(cmd_err)
		ret

;------------------------------------------------------------------------------
;--- compare flash string to ram string argument 
;--- z points to flash, y to ram, carry set on success, cleared if no match
;--- trash temp temp2,  z exits @ failing char
cmp_fz2ry:	push	yh		; save our ram pointer	
		push	yl
		
cmp_fy2rz_lp:	ld	temp,y+		; get ram
		lpm	temp2,z+	; get flash
		
		tst	temp2		; flash string is done (a match)	
		brne	cmp_cp

		tst	temp		; we want to be sure ram argument ends in a null ...
		breq	cmp_fy2rz_suceed	
		
		cpi	temp,' '	; or space char
		breq	cmp_fy2rz_suceed

		rjmp	cmp_fy2rz_fail	
		
cmp_cp:		rcall	to_upper
		cp	temp,temp2
		brne	cmp_fy2rz_fail	
	
		rjmp	cmp_fy2rz_lp
		
cmp_fy2rz_fail:	clc
		sbiw	zl,1		; adjust for last z+
		rjmp	pc + 2	
cmp_fy2rz_suceed:
		sec
		pop	yl
		pop	yh
		ret	 

;------------------------------------------------------------------------------
;--- init eeprom to default flash stored values, command not implemented yet
restore_ee_2_defaults:
		_init_eeaddr	0

ee_clr_lp:	wdr
		clr	temp		; write zeros to all 512 eeprom locations	
		rcall	eewr_p		; write with post inc of eeaddr

		mov	temp,eeaddr_h
		cpi	temp,0x02	; test for rollover from 0x1ff to 0x200			
		brne	ee_clr_lp		

		_init_eeaddr	_mycallsign
		_init_zflash	_default_call

		ldi	temp2,16		; 16 bytes to transfer (gets unproto and mycall)
		rcall	flash2ee	

		_init_eeaddr	_loc_call
		_init_zflash	_default_loc_call
		ldi	temp2,7			; 7 bytes to transfer 
		rcall	flash2ee	

		_init_eeaddr	_nv_flags_ee
		_init_zflash	_factory_vals
		ldi	temp2,16		; 16 bytes to transfer 
		rcall	flash2ee	
		ret

;------------------------------------------------------------------------------
;--- debugging routines, 
;--- we muck with this routine a lot depending on what we are trying to debug
;--- don't call from isr! (odd results)
debug_dump:	sts	_debug_temp,temp			; save ram copy of temp
		in	temp,sreg				; save status reg
		push	temp

		push	zl
		push	zh

		rcall	crlf
		_send_flash	_debug_string
		rcall	crlf

		clr	zh
		clr	zl
 		
diag_lp:	lds	temp,_debug_temp			; dumb, but necessary

		ld	temp,z+
		rcall	send_byte
		_send	' '

		cpi	zl,26
		brne	diag_lp		

		ldd	temp,z+1				; x
		rcall	send_byte
		ld	temp,z					; swapping h and l for display purposes
		rcall	send_byte
		_send	' '
		
		ldd	temp,z+3				; y
		rcall	send_byte
		ldd	temp,z+2
		rcall	send_byte
		_send	' '
		
		pop	temp					; zh
		rcall	send_byte
		pop	temp					; zl
		rcall	send_byte
		_send	' '
			
		pop	temp					; get sreg
		push	temp					; save it again	
		rcall	send_byte
		rcall	crlf

		rcall	dump_ram
		
		pop	temp					; get sreg
		out	sreg,temp				; restore it
		lds	temp,_debug_temp			; restore temp

		ret

;--- debugging
_debug_string:	.db	"00 01 02 03 04 05 06 07 08 09 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 XhXl YhYl ZhZl SR",0


;--- stub to dump ram as console command
dump_sram:	rcall	dump_ram
		rjmp	prompt		

;----------------------------------------------------------------------
;--- display ram contents on console (0x00 to 0xff)
dump_ram:	push	zh
		push	zl
		push	temp
		ldi	zl,0x00
		clr	zh

dump_l_cr:	rcall	crlf
dump_loop:	mov	temp,zl
		rcall	send_byte
		rcall	send_eq
		ld	temp,z+
		rcall	send_byte
		_send	' '

		cpi	zl,(0x100)	
		breq	dump_s_done

		ldi	temp,0x0f
		and	temp,zl
		breq	dump_l_cr
		rjmp	dump_loop		

dump_s_done:	rcall	crlf
		pop	temp
		pop	zl
		pop	zh	
		ret

;-----------------------------------------------------------------
;--- display first page of eeprom contents on console 
dump_ee:	clr	ii
		_init_eeaddr	0x00
dump_ee_cr:	rcall	crlf			; formatting
dump_ee_loop:	mov	temp,ii
		rcall	send_byte
		rcall	send_eq
		rcall	eerd_p	
		rcall	send_byte
		_send	' '
		inc	ii
		cpi	ii,0x00
		breq	dump_ee_done
		ldi	temp,0x0f
		and	temp,ii
		breq	dump_ee_cr
		rjmp	dump_ee_loop	
dump_ee_done:	rjmp	prompt

		
;*********************************************************************
; codespace string storage / cmd table
;*********************************************************************
;--- note: if the routine name does not exist, avrstudio will not generate an error! (you'll get weird program behavior)

_cmd_table:		; arg string, 0, routine address
_auxlink:	.db	"AUXLINK",0,high(aux_link),low(aux_link)
_axlf:		.db	"AXLF",0,high(set_axlf),low(set_axlf)	 	
_baud:		.db	"BAUD",0,high(get_baud),low(get_baud)
_beacon:	.db	"BEACON",0,high(get_btime),low(get_btime)
_bootloader:	.db	"BOOTLOADER",0,high(enter_bootloader),low(enter_bootloader)	
_btext:		.db	"BTEXT",0,high(get_btext),low(get_btext)			
_cal:		.db	"CAL",0,high(cal_routines),low(cal_routines)
_converse:	.db	"CONVERS",0,high(set_converse),low(set_converse)		 
_debug:		.db	"DEBUG",0,high(get_debug),low(get_debug)			
_dump: 		.db     "DUMP",0,high(dump),low(dump)		     	 	
_echo:		.db	"ECHO",0,high(set_echo),low(set_echo)			
_ee:		.db	"EE",0,high(dump_ee),low(dump_ee)
_fixtype:	.db	"FIXTYPE",0,high(get_fixtype),low(get_fixtype)				
_gpsinit:	.db	"GPSINIT",0,high(send_gps_init),low(send_gps_init)	
_gpsistr:	.db	"GPSISTR",0,high(get_gpsistr),low(get_gpsistr)			
_gpsq:		.db	"GPSQ",0,high(gps_query),low(gps_query)
_header:	.db	"HEADER",0,high(set_header),low(set_header)	
_help:		.db	"?",0,high(disp_commands),low(disp_commands)			
_k1:		.db	"K1",0,high(k1_routine),low(k1_routine)		
_k:		.db	"K",0,high(set_converse),low(set_converse)			
_lf:		.db	"LF",0,high(set_lf),low(set_lf)	
_lpath:		.db	"LPATH",0,high(get_lpath),low(get_lpath)
_ltext:		.db	"LTEXT",0,high(get_ltext),low(get_ltext)
_ltime:		.db	"LTIME",0,high(get_ltime),low(get_ltime)			
_mcom:		.db	"MCOM",0,high(set_mcom),low(set_mcom)		
_monitor:	.db	"MONITOR",0,high(get_monitor),low(get_monitor)
_mycall:	.db     "MYCALL",0,high(get_mycall),low(get_mycall)			
_persist:	.db	"PERSIST",0,high(get_persist),low(get_persist)		
_pwrupconv:	.db	"PWRUPCONV",0,high(set_pwrup_conv_mode),low(set_pwrup_conv_mode)		
_reset:		.db	"RESET",0,high(reset_tnc),low(reset_tnc)	
_restore:	.db	"RESTORE",0,high(restore_tnc),low(restore_tnc)
_rmcexpire:	.db	"RMCEXPIRE",0,high(get_rmc_expire),low(get_rmc_expire)
_slot:		.db	"SLOT",0,high(get_slot),low(get_slot)			
_symbol:	.db	"SYMBOL",0,high(get_symbol),low(get_symbol)
_trace:		.db	"TRACE",0,high(get_trace),low(get_trace)			
_txdelay:	.db	"TXDELAY",0,high(get_txdelay),low(get_txdelay)
_unproto:	.db	"UNPROTO",0,high(get_unproto),low(get_unproto)
_ver:		.db	"VER",0,high(disp_ver),low(disp_ver)					
_cmds_end:	.db	 0,0 ; mark end of table

	
;--- message strings----------------
_hello:		.db	cr,lf,"ATmega-TNC Version 1.8 Copyright 2003 Henry Carl Ott N2RVQ",cr,lf,0
_eh:		.db	"EH ?",0
_via:		.db	" VIA ",0
_error:		.db	"-ERROR-",0  
_on:		.db	"ON",0
_off:		.db	"OFF",0   
_seconds:	.db	"SECONDS",0
_cmd:		.db	"cmd:",0
_wx:		.db	"WX>",0
_gps:		.db	"GPS>",0
_filter_string:	.db	"$GPRMC",0		; used to filter rmc strings  


;--- baud to ubbr translation
_baud_ubbr:	.db	(clock - 8 * 2400) / (16 * 2400),(clock - 8 * 4800) / (16 * 4800)
_baud_ubbr_1:	.db	(clock - 8 * 9600) / (16 * 9600),(clock - 8 * 19200) / (16 * 19200)
_baud_ubbr_2:	.db	(clock - 8 * 38400) / (16 * 38400),(clock - 8 * 57600) / (16 * 57600)
_baud_ubbr_3:	.db	(clock - 8 * 76800) / (16 * 76800),(clock - 8 * 115200) / (16 * 115200)

; data used to init eeprom to default values
_default_loc_call:				; same as default call	
_default_call:	.db	"NOCALL",0x30,0
_default_unp:	.db	"UNPROT",0x30,0
_factory_vals:	.db (0<<echo)|(0<<pwrupconv)|(1<<lf_en)|(0<<header)|(0<<mcom)|(0<<ax_lf_en),0x00,0x00,0x00,0x00,0x00,0x00,2,10,63,40,0,'/','-'
		;   nv flags,b_timer_hi,b_timer_lo,p_timer_hi,p_timer_lo,t_timer_hi,t_timer_lo,baud_div,slot,persist,txdelay,fixtype,symbol table,symbol                               

;*********************************************************************
; EEPROM 
;*********************************************************************
.eseg
.org		0x00
_empty:		.db 	0x00			; we leave empty, may possibly get corrupted  
;--- mycall and dest unproto should be 6 chars followed by ssid (add 30 hex) and then zero terminated
_mycallsign:	.db	"NOCALL",0x30,0		; my callsign 
_unp_call:	.db     "UNPROT",0x30,0        	; destination callsign
_unp_path:	.db	0,0,0,0,0,0,0		; digi list, terminate last call with zero
		.db	0,0,0,0,0,0,0
		.db	0,0,0,0,0,0,0
		.db	0,0,0,0,0,0,0
		.db	0,0,0,0,0,0,0
		.db	0,0,0,0,0,0,0,0

;--- leave room for up to 6 digis 7 chars per, plus a trailing zero 

_loc_call:	.db     "NOCALL",0x30,0        	;  destination of location packets
_loc_path:	.db	0,0,0,0,0,0,0		; digi list, terminate last call with zero
		.db	0,0,0,0,0,0,0
		.db	0,0,0,0,0,0,0
		.db	0,0,0,0,0,0,0
		.db	0,0,0,0,0,0,0
		.db	0,0,0,0,0,0,0,0

.org 		0xff
		.db	0x00				
.org		0x100
_gpsinit_text_ee:.db	0			; 48 chars
.org		0x130
_ltext_ee:	.db	0			; 48 chars
.org		0x160
_btext_ee:	.db	0			; 64 chars


.org		0x1f0
_nv_flags_ee:	.db	(0<<echo)|(0<<pwrupconv)|(1<<lf_en)|(0<<header)|(0<<mcom)|(0<<ax_lf_en) ; non volitile flag storage
						; 16 bit timers, keep them in order
_rmc_expire_ee:	.db	0x00,0x00		; 
_btime_ee:	.db	0x00,0x00		; beacon timer		
_ltime_ee:	.db	0x00,0x00		; location timer reload value	

_baud_ee:	.db	2			; default 9600
_slot_ee:	.db	10
_persist_ee:	.db	63
_tx_delay_ee:	.db	40
_fix_type_ee:	.db	0			; 0=aprs 1=rmc
_pos_table_char:.db	'/'
_pos_sym_char:	.db	'-'

