;***********************************************
;* This is a modified version of design note #032 'DN032' From http://www.avrfreaks.com
;* It is meant specifically for atmega-8 and avrprog.exe 1.37 or better.
;* The original DN did not seem to work properly with a atmega8, after some debugging it 
;* appeard that the problem was that the code did not re-enable RWW access after page write or erase. Fixed.
;* Notes:
;* Added an esc char test at the beginning of the code. Modify as necessary.
;* Be sure to use avrprog.exe 1.37 or a version that supports atmega8 (the regular download from atmel is 1.33)
;* While debugging I also discovered that avrprog support a baud rate of 115200, seems to work okay.
;* Fuses: BLB11 BOOTSZ1 BOOTRST
;* comments or questions?  carlott@si.rr.com   http://users.rcn.com/carlott/

;********** B O O T L O A D E R **********
;* File : bootloader_m8.asm (Include chip erase counter)
;* Compiler : AVR Studio
;* Target : mega8


;--- device code and signature bytes for a atmega8 with bootloader
.equ 	DT  = 0x77	; atmega8 bootloader
.equ 	SB1 = 0x07 	; Signature byte 1
.equ 	SB2 = 0x93 	; Signature byte 2
.equ 	SB3 = 0x1e 	; Signature byte 3

;.equ	clock = 7372800
.equ	bootldr_baud	= 115200		; only valid baud rates for avrprog.exe are 19200 and 115200


.CSEG
.org		SECONDBOOTSTART	 

boot_reset:

;--- init stack
		ldi 	R24,low(RAMEND)
		ldi 	R25,high(RAMEND)
		out 	SPL,R24
		out 	SPH,R25 			; SP = RAMEND
;--- init uart
		ldi 	R24,(clock - 8 * bootldr_baud) / (16 * bootldr_baud) 	; Baud rate 
		out	UBRRL,R24
		ldi 	R24,(1<<RXEN)|(1<<TXEN)
		out 	UCSRB,R24 			; Enable receiver & transmitter, 8-bit mode

;--- following waits for esc char, if not received within a given period, jump to application code 
;--- about 1.5 seconds 	@ 7.3728 clock 
;--- delete or chose your own way of entering bootstrap code

		clr	R23
		clr	R24
		clr	R25

boot_esc_lp:	wdr
		dec	R23
		brne	boot_r_1
		dec	R24
		brne	boot_r_1	
		inc	R25
		cpi	R25,30
		brne	boot_r_1
		rjmp	0x0000				; timeout, jump to normal reset vector
boot_r_1:
		sbis 	UCSRA,RXC 			; wait for incoming data (until RXC==1)
		rjmp 	boot_esc_lp
		in	R16,UDR 			; return received data in R16
		cpi	R16,0x1b
		brne	boot_esc_lp
		rjmp	boot_id				; escaped rcvd, dosplay id and loop to main

bootloadermain:	rcall 	uartGet 			; repeat (R16 = uartGet)
		cpi 	R16,0x1B			; while (R16 == ESCAPE)
		breq 	bootloadermain

		cpi	R16,'a'				; if(R16=='a') 'a' = Autoincrement?
		brne 	L12
		ldi 	R16,'Y' 			; Yes, autoincrement is quicker
		rjmp 	L70 				; uartSend(R16)

;--- get address, convert to word address
L12: 		cpi 	R16,'A' 			; else if(R16=='A') write address
		brne 	L14
		rcall 	uartGet
		mov 	R27,R16 			; R27 = address high byte
		rcall 	uartGet
		mov 	R26,R16 			; R26 = address low byte
		lsl 	R26 				; address=address<<1
		rol 	R27 				; convert from byte address to word address
		rjmp 	send_cr 			; uartSend('\r')

;---  get program data low byte 
L14: 		cpi 	R16,'c' 			; else if(R16=='c') write program memory, low byte
		brne 	L16
		rcall 	uartGet
		mov 	R22,R16 			; R22 = data low byte
		rjmp 	send_cr 			; uartSend('\r')

;---  get program data high byte, and store in temp buffer 
L16: 		cpi 	R16,'C' 			;else if(R16=='C') write program memory,high byte
		brne 	L18

		rcall 	uartGet
		mov 	R23,R16 			; R23 = data high byte

		movw 	R30,R26 			; Z pointer = address
		movw 	R0,R22 				; R0&R1 = data
	
		ldi 	R24,1 				; SPMCR = 0x01
		out 	SPMCR,R24			; page load (fill temporary buffer)
		spm 					; Store program memory

		adiw 	R26,2 				; address=address+2
		rjmp 	send_cr 			; uartSend('\r')

;--- chip erase
L18: 		cpi 	R16,'e' 			; else if(R16=='e') Chip erase
		brne 	L28
		
; for(address=0; address < (2*SECONDBOOTSTART); address += (2*PAGESIZE))

		clr 	R26 				; page_erase();
		clr 	R27
		rjmp	L24				; test for end

L20: 		movw 	R30,R26 			; Z-pointer = address
		ldi 	R16,3 				; SPMCR = 0x03
		rcall	wait_spm			; spm, wait for done
	
		subi 	R26,low(-2*PAGESIZE) 		; address += (2*PAGESIZE)
		sbci 	R27,high(-2*PAGESIZE)

L24: 		ldi 	R24,low(2*SECONDBOOTSTART)
		ldi 	R25,high(2*SECONDBOOTSTART)
		cp 	R26,R24 			; address < Boot Flash address(byte address) 0x3E00 ?
		cpc 	R27,R25
		brlo 	L20
	
		rjmp 	send_cr				; uartSend('\r')

;--- page write
L28: 		cpi 	R16,'m' 			; else if(R16== 'm') Write page
		brne 	L34
		movw 	R30,R26 			; Z-pointer = address
		ldi 	R16,5 				; SPMCR = 0x05 Write page
		rcall	wait_spm			; spm , wait for done
L32: 		rjmp 	send_cr 			; uartSend('\r')
				

;--- enter programming mode
L34:		cpi 	R16,'P' 			; else if(R16=='P') Enter programming mode
		breq 	L32 				; uartSend('\r')
		cpi 	R16,'L' 			; else if(R16=='L') Leave programming mode
		breq 	L32 				; uartSend('\r')

;--- return programing type
		cpi 	R16,'p' 			; else if (R16=='p') Return programmer type
		brne 	L38

		ldi 	R16,'S' 			; uartSend('S') Serial
		rjmp 	L70 				; uartSend(R16)

;--- read program memory
L38: 		cpi 	R16,'R' 			; else if(R16=='R') Read program memory
		brne 	L40

		movw 	R30,R26 			; Z-pointer <= address

		lpm 	R24,Z+ 				; read program memory LSB; store LSB in R24 and Z pointer ++
		lpm 	R16,Z+ 				; read program memory MSB; store MSB in R16 and Z pointer ++
		rcall 	uartSend 			; uartSend(R16) MSB

		movw 	R26,R30 			; address += 2
		mov 	R16,R24 			; LSB stored in R16
		rjmp 	L70 				; uartSend(R16) LSB


;--- eeprom write
L40: 		cpi 	R16,'D' 			; else if (R16=='D') Write data to EEPROM
		brne 	L42

		rcall 	uartGet
		out 	EEDR,R16 			; EEDR = uartGet()

		ldi 	R17,6 				; write EEPROM
		rcall 	EepromTalk
		rjmp 	send_cr 			; uartSend('\r')

;--- eeprom read
L42: 		cpi 	R16,'d' 			; else if (R16=='d') Read data from EEPROM
		brne 	L44

		ldi 	R17,1 				; read EEPROM
		rcall 	EepromTalk 			; R16 = EEPROM data
		rjmp 	L70 				; uartSend(R16)

;--- read fuse bits
L44: 		cpi 	R16,'F' 			; else if(R16=='F') Read fuse bits
		brne 	L46

		clr 	R30 				; Z-pointer = 0000
		rjmp 	L50 				; rcall readFuseAndLock

;--- read lock bits
L46: 		cpi 	R16,'r' 			; else if(R16=='r') Read lock bits
		brne 	L48

		ldi 	R30,1 				; Z pointer = 0001
		rjmp 	L50 				; rcall readFuseAndLock

;--- read high fuse bits
L48: 		cpi 	R16,'N' 			; else if(R16=='N') Read high fuse bits
		brne 	L52
		ldi 	R30,3 				; Z-pointer = 0003

;--- read fuse and lock bits
L50: 		rcall 	readFuseAndLock
		rjmp	L70 				; uartSend(R16)

;--- return supported devices
L52:	 	cpi 	R16,'t' 			; else if(R16=='t') Return supported devices code
		brne 	L54

		ldi 	R16,DT 				; Device Type
		rcall 	uartSend 			; uartSend(DT) send Device Type

		clr 	R16
		rjmp 	L70 				; uartSend(0)

;---  grab another char and ignore
L54: 						; else if ((R16=='l')||(R16=='x')||(R16=='y')||(R16=='T'))
		cpi 	R16,'l' 			; 'l' = Write Boot Loader lockbits 
		breq 	L56
		cpi 	R16,'x' 			; 'x' = Set LED
		breq 	L56
		cpi 	R16,'y' 			; 'y' = Clear LED
		breq 	L56

		cpi 	R16,'T' 			; 'T' = Select device type
		brne 	L60


L56: 		rcall 	uartGet 			; R16 = uartGet()
; YOU CAN INSERT LEDs CODE HERE
		rjmp 	send_cr				; uartSend('\r')

;--- send return software id
L60: 		cpi 	R16,'S' 			; else if (R16=='S') Return software identifier
		brne 	L62

boot_id:	ldi 	R30,low(2*Soft_Id)
		ldi 	R31,high(2*Soft_Id)
L61: 		lpm 	R16,Z+
		tst 	R16
		breq 	L72 				; branch is end of string ((Z) == 0)
		rcall 	uartSend 			; else send char
		rjmp 	L61

;--- send version
L62: 		cpi 	R16,'V' 			; else if (R16=='V') Return Software Version
		brne 	L64

		ldi 	R16,'1' 			; uartSend('1')
		rcall 	uartSend
		ldi 	R16,'2' 			; uartSend('2')
		rjmp 	L70 				; uartSend(R16)

;--- send signature bytes
L64: 		cpi 	R16,'s' 			; else if (R16=='s') Return Signature Byte
		brne 	L66

		ldi 	R16,SB1 			; uartSend(SB1) Signature Byte 1
		rcall 	uartSend
		ldi 	R16,SB2 			; uartSend(SB2) Signature Byte 2
		rcall 	uartSend
		ldi 	R16,SB3 			; uartSend(SB3) Signature Byte 3
		rjmp 	L70 				; uartSend(R16)

L66:		ldi	R16,0x3f 			; else uartSend('?')
		rjmp 	L70 				; uartSend(R16)

;--- send cr
send_cr: 	ldi 	R16,13 				; uartSend('\r')

;--- send byte via uart
L70: 		rcall 	uartSend 			; uartSend(R16)
L72: 		rjmp 	bootloadermain

readFuseAndLock:
		clr 	R31 				; Z pointer high byte = 0
		ldi 	R24,9 				; SPMCR = 0x09
		out 	SPMCR,R24 			; read fuse and lock
		lpm 	R16,Z 				; read program memory
		ret

EepromTalk:		 				; if R17 == 6 then Write, if R17 == 1 then Read
		out 	EEARL,R26 			; EEARL = address low
		out 	EEARH,R27 			; EEARH = address high
		adiw 	R26,1 				; address++
		sbrc 	R17,1 				; skip if R17 == 1 (read Eeprom)
		sbi 	EECR,EEMWE 			; EEMWE = 1 (write Eeprom)
		out 	EECR,R17 			; EECR = R17 (6 write, 1 read)
L90:	 	sbic 	EECR,EEWE 			; wait until EEWE == 0
		rjmp 	L90
		in 	R16,EEDR 			; R16 = EEDR
		ret

uartSend: 						; send R16
		sbis 	UCSRA,UDRE 			; wait for empty transmit buffer (until UDRE==1)
		rjmp 	uartSend
		out 	UDR,R16 			; UDR = R16, start transmission
		ret

uartGet:	wdr
		sbis 	UCSRA,RXC 			; wait for incoming data (until RXC==1)
		rjmp 	uartGet
		in	R16,UDR 			; return received data in R16
		ret

wait_spm:	out 	SPMCR,R16
		spm 
wait_spm_lp:	in	R16,SPMCR
		sbrc	R16,SPMEN
		rjmp	wait_spm_lp
		ldi	R16,(1<<RWWSRE) | (1<<SPMEN)
		out	SPMCR,R16
		spm
		ret 

Soft_Id: .DB "AVR-M8B", 0