;***************************************************************************
;* Title: timer one int and support sub routines
;* the isr handles the low level transmission of ax25 bytes and assembles received packets into ax25_rx_buffer 
;* also handles system timers, 
;*
;* Made available for educational, and non-commercial use.
;* Commercial use forbidden without license from author.
;* Code may be modified as long as it is released publicly and credit
;* is given to the original author(s).  
;* (c) copyright 2002 Henry Carl Ott N2RVQ, all Rights reserved.
;* Questions, comments or offers of cash? carlott@si.rr.com  http://users.rcn.com/carlott/
;***************************************************************************

;***************************************************************************
;--- timer 1 overflow interupt, we should hit the isr @ 9600 hz		
t1int_isr:	in	isr_save,SREG			; save SREG
		push	temp		
		push	zl				; we have to use zl and zh for the jump table anyway
		push	zh				; so we might as well use the for gp 
			
		ldi	zh,high(isr_reload_val)		; reload timer1
		ldi	zl,low(isr_reload_val)
		out	tcnt1h,zh
		out	tcnt1l,zl

;--- ax25 transmit portion of isr
ax25_tx:	mov	temp,ax25_tx_counts		; get tx bit cnt, anything to send?	
		andi	temp,0b00001111
		breq	ax25_tx_end			; no, branch to end

		lds	temp,_tic			; 9600 divided by 8, see if we are in a tx bit period (1200 baud)
		andi	temp,0x07			; mask off hi bits	
		brne	ax25_tx_end			; nope, fall through to rx	

		mov	temp,ax25_tx_counts		; yes we have data
		andi	temp,0b01110000
		cpi	temp,0x50			; see if it's necessary to stuff a zero
		brne	send_bit			; nope, keep going

		sbrs	ax25_flags,tx_bit_stuff	; test if bit stuffing is enabled 
		rjmp	send_bit 			; nope		

		rcall	ax25_tx_0			; stuff a zero and exit		
		rjmp	ax25_tx_end			; 

send_bit:	dec	ax25_tx_counts			; dec bit counter, only if a bit is sent	
		ldi	temp,1<<ones_cnt0
		add	ax25_tx_counts,temp		; inc ones counter, tx_0 will zero it anyway (ones counter is in high nibble)

		sbrs	ax25_txbuff,0			; test lsb,  data is sent lsb first
		rcall	ax25_tx_0

		lsr	ax25_txbuff			; shift next bit into tx position, bit just sent goes into carry
			
ax25_tx_end:
;--- ax25 rx portion of isr
ax25_rx:	sbrs	ax25_flags,rx_enable		; is reception enabled?
		rjmp	ax25_rx_end			; no, jump to end	
	
		sbis	MX_INP,DET			; any dcd?
		rjmp	rx_flush			; nope, reset all 

		tst	ax25_rx_interval			; test transition counter
		breq	pc + 2				; if @ zero, stay at zero
		dec	ax25_rx_interval

		in	temp,MX_INP			; get copy of pin
		bst	temp,MX_RXD			; save a copy of the bit			

		eor	temp,ax25_flags	
		sbrs	temp,bit_sample			; test for change
		rjmp	ax25_rx_end			; nope, no edge detected, keep rolling, do nothing

rx_change:	bld	ax25_flags,bit_sample	; save new bit status for next time 

		tst	ax25_rx_interval			; test for timout
		brne	pc + 2				; > 0 ?
		rjmp	rx_flush			; too long between transitions, timeout, dump all 

		lsr	ax25_rx_interval
		lsr	ax25_rx_interval
		lsr	ax25_rx_interval			; divide count by 8

		_init_z	rx_jmp_tbl

		add	zl,ax25_rx_interval
		adc	zh,zero				; if a carry from low order add it in
		ijmp
		
;--- jump table
rx_jmp_tbl:	rjmp	rx_flag			;0 flag
		rjmp	rx_add_5		;1 zero stuff, add five ones toss zero
		rcall	rx_add_1		;2 11110
		rcall	rx_add_1		;3 1110
		rcall	rx_add_1		;4 110
		rcall	rx_add_1		;5 10
		rjmp	rx_add_0		;6 0
		rjmp	rx_flush 		; error, too soon


;--- add 5 ones to buffer, and discard extra stuffed zero
rx_add_5:	rcall	rx_add_1				; only one more instruction then a loop
		rcall	rx_add_1				; is faster, and we don't have a spare loop register 
		rcall	rx_add_1
		rcall	rx_add_1
		rcall	rx_add_1
		rjmp	rx_reload_cnt				; done		

;--- add a single one to buffer, carry contains bit
rx_add_1:	sec						; set carry
		rcall	rx_in_bit				; rotor it in
		ret

;--- add a zero to receive  buffer
rx_add_0:	clc						; clear carry	
		rcall	rx_in_bit				; rotor in
		rjmp	rx_reload_cnt				; done, exit and wait for next edge	


;--- flag detected, if in packet try to close, else discard, reset counters 
rx_flag:	sbr	ax25_flags,(1 << flag_det)		; set flag detected flag
		sbrs	ax25_flags,in_packet			; are we currently within a packet? then try to close
		rjmp	rx_flag_flush				; not in packet, reset counters, leave flag bit set


;--- try to close packet and reset for next, discard bad packets
;--- the constant we compare the crc to is what you get when you clock the whole packet through the crc routine (including the fcs)
ax25_rx_close:	lds	temp,_ax25_rx_cnt		; get count

		cpi	temp,min_packet_len		; minimum length packet
		brlo	rx_flag_flush			; too small, discard

		lds	temp,_rx_crc_hi
		cpi	temp,0xf0			; compare with calculated crc 
		brne	rx_flag_flush			; bad crc

		lds	temp,_rx_crc_lo
		cpi	temp,0xb8			; compare with calculated crc 
		brne	rx_flag_flush			; bad crc

		lds	temp,_ax25_rx_cnt		; reduce to discard crc
		subi	temp,2		
		sts	_ax25_rx_cnt,temp		; and save length

		led1_on
		sbr	ax25_flags,1<<packet_rx		; set flag to indicate packet reception
		rjmp	rx_flag_flush			; reset, for next packet but leave flag bit alone

rx_flush:	cbr	ax25_flags,(1 << flag_det)	; flush all
rx_flag_flush:	cbr	ax25_flags,(1 << in_packet)	; reset flags,
		clr	ax25_rx_bitcnt			; clear bit counter

		ldi	temp,0xff			; reset rx crc
		sts	_rx_crc_lo,temp
		sts	_rx_crc_hi,temp

rx_reload_cnt:	ldi	temp,reload_count		; get ready for next bit transistion
 		mov	ax25_rx_interval,temp		; reload edge counter 

ax25_rx_end:
system_timers:	lds	temp,_tic			; dec counter everytime through	
		subi	temp,1				; divide by 96 to get our 10ms tic	
		brcc	pc + 2	
		ldi	temp,95
		sts	_tic,temp
		brcc	no_rollover	

		lds	temp,_ms_10	
		subi	temp,1
		brcc	pc + 2
		ldi	temp,99
		sts	_ms_10,temp			; bump at 100 hz
		brcc	no_rollover
	
		sbr	sys_flags,1<<_1hz		; set ihz flag bit
		sbis	CTS_PORT,CTS			; only inc 232timer if cts is de-asserted
		rjmp	rmc_exp_timer

inc_sec_timers:	lds	temp,_rs232_timer		; counts up if cts de-asserted (pin hi)
		inc	temp
		sts	_rs232_timer,temp

rmc_exp_timer:	_init_z	_rmc_exp_timer
		rcall	inc16_at_z			; inc rmc exire timer

btimer_inc:	rcall	inc16_at_z			; inc btimer	
ltimer_inc:	rcall	inc16_at_z			; be sure these timers are stored in ram in order

no_rollover:	pop	zh				; restore
		pop	zl				; 
		pop	temp				; 
		out	SREG,isr_save			; restore
		reti

;--- rotates carry into rx buffer, when rx_buff is full, store to ram if room
;--- if a complete byte and last byte was a flag, start packet reception (set flags) 
rx_in_bit:	ror	ax25_rxbuff 			; save bit rotate carry into position (lsb first)	

		inc	ax25_rx_bitcnt			; 8 bits to a byte  
		sbrs	ax25_rx_bitcnt,3		; store byte
		ret					; below 8 just return		

		clr	ax25_rx_bitcnt			; reset bit counter

		sbrc	ax25_flags,in_packet
		rjmp	store_ax_byte			; within a packet, handle byte

		sbrs	ax25_flags,flag_det		; not in packet, was the last byte a flag?	
		ret					; nope, just discard 

start_packet:	sbrc	ax25_flags,packet_rx		; was last packet processed?
		rjmp	packet_over_run			; no 

		sbr	ax25_flags,(1<<in_packet) 	; start of packet data (that's the  theory)
		cbr	ax25_flags,(1<<flag_det)	; clear flag detect
		sts	_ax25_rx_cnt,zero		; zero char counter
		
store_ax_byte:	push	yh					
		push	yl				; work space

		lds	yh,_rx_crc_hi			; get crc from ram
		lds	yl,_rx_crc_lo
				
		ldi 	zh,0x84				; preload 16 bit poly for speed 	
		ldi 	zl,0x08 			; 
		eor 	yl,ax25_rxbuff 			; xor in byte

		ldi	temp,8				; 8 bits

crc_rx_lp:	lsr 	yh				; loop through all 8 bits
		ror 	yl
		brcc 	crc_rx_1
		eor 	yl,zl
		eor 	yh,zh
crc_rx_1:	dec	temp
		brne	crc_rx_lp
			
		sts	_rx_crc_hi,yh			; save results
		sts	_rx_crc_lo,yl

		pop	yl				; restore work space
		pop	yh

		lds	temp,_ax25_rx_cnt		; get number of chars received
		cpi	temp,max_rx_packet_len		; any more room?				
		brsh	store_ax_end			; no room in buffer, discard char

		_init_z	_ax25_rx_buffer			; init pointer to data

		add	zl,temp				; add in offset	
		adc	zh,zero
		st	z,ax25_rxbuff			; save data in buffer 

		inc	temp				; bump byte counter
		sts	_ax25_rx_cnt,temp
store_ax_end:	ret

;-----------------------------------------------------------------
;--- start of new packet data before buffer was emptied
packet_over_run:
		sbr	ax25_flags,(1<<over_run)	; an overrun, clear flag bit, and discard byte
                cbr	ax25_flags,(1<<flag_det)	;
		ret	
;-------------------------------------------------------------------
;--- flip mx614 txd line line, zero the ones counter
;--- trashes temp,zl
ax25_tx_0:	in	temp,MX_PORT			; get existing port data (output latch)
		ldi	zl,1<<MX_TXD			; get pin position
		eor	temp,zl				; flip it

		out	MX_PORT,temp			; backout to latch

		ldi	temp,0b10001111
		and	ax25_tx_counts,temp		; clear ones counter
		ret	

;-------------------------------------------------------------------
;--- inc 16 bit value at z low byte first
inc16_at_z:	ld	temp,z 				; location timer
		subi	temp,-1				; add 1
		st	z+,temp

		brne	inc_z_end	
		ld	temp,z	 
		subi	temp,-1				
		st	z,temp
inc_z_end:	adiw	zl,1				; adjust pointer
		ret	