;***************************************************************************
;* Title: uart data reception and transmission routines
;* isr/non isr hardware primary, software bit bang using timer for aux port, and wx 
;* circ buffers for primary, linear buffers for aux, partial handshaking on main 
;* 
;* Made available for educational, and non-commercial use.
;* Commercial use forbidden without license from author.
;* Code may be modified as long as it is released publicly and credit
;* is given to the original author(s).  
;* (c) copyright 2003 Henry Carl Ott N2RVQ, all Rights reserved.
;* Questions, comments or offers of cash? carlott@si.rr.com  http://users.rcn.com/carlott/
;***************************************************************************
;--- 2400 baud wx receive support
;--- comm port support (low baud rate) transition on int1 pin starts us up, then use timer int
int1_isr:	in	isr_save,SREG			; Store Status Register
		push	temp

		ldi	temp, 256 - (wx_bit_reload + (wx_bit_reload/2))	
		out	TCNT2,temp			; try to put us into the middle of the first data bit		
						
		ldi	temp,(1<<TOV2)			;  
		out	TIFR,temp			; to clear any existing timer 0 overflow flag

		in	temp,TIMSK			; get current interupt enable mask
		sbr	temp,(1 << TOIE2)		; 
		out	TIMSK,temp			; enable Timer 0 overflow interrupt

		in	temp,GICR			; get current int enables
		cbr	temp,1<<INT1			; 
		out	GICR,temp			; disable external interrupt

		ldi	temp,0xf0
		and	wx_uart_status,temp		; clear bit counter 

		pop	temp
		out	SREG,isr_save			; Restore SREG
		reti

;------------------------------------------------------
;--- timer 2 overflow, wx uart support, build received bits to bytes   
t2int_isr:	in	isr_save,SREG			;  store SREG
		push	temp

		ldi	temp,256 - wx_bit_reload
		out	tcnt2,temp

		inc	wx_uart_status	

wx_rcv_bit:	mov	temp,wx_uart_status
		andi	temp,0x0f			; mask off busy bit

		cpi	temp,9				; check for stop bit
		breq	wx_got_char

		clc					; not stop bit, rotate in bit, keep rolling
		sbic	WX_INP,WX_RXD			; copy input pin to carry
		sec					;
		ror	wx_char_buff			; shift carry bit into data

 		rjmp	t2_int_end			; wait for next timer int

wx_got_char:	sbrc	wx_uart_status,wx_cr_rcvd	; was last string processed?
		rjmp	wx_out				; no, toss char
		
		mov	temp,wx_char_buff
		cpi	temp,lf				; test for lf
		breq	wx_out				; toss char
	
		cpi	temp,cr				; test for cr
		brne	wx_store_char	

		ldi	temp,(1 << wx_cr_rcvd)		; 	
		or	wx_uart_status,temp		; set flag bit
		rjmp	wx_out

wx_store_char:	lds	temp,_wx_rx_cnt			; get current char count
		cpi	temp,wx_rx_len
		brsh	wx_out				; too long, toss char		

		push	zl				; save registers
		push	zh		
		_init_z	_wx_rx_buffer			; get location

		add	zl,temp
		adc	zh,zero
		st	z,wx_char_buff			; save char
	
		inc	temp				; inc char counter
		sts	_wx_rx_cnt,temp			; save it

		pop	zh				; restore work register	
		pop	zl				; restore work register

wx_out:		in	temp,TIMSK 
		cbr	temp,(1 << TOIE2)		; mask off bit
		out	TIMSK,temp	    		; disable timer 1 int
		
		ldi	temp,0xf0
		and	wx_uart_status,temp		; clear bit counter 

		ldi	temp,(1 << INTF1)		; clear any pending edge flag by writing a one to it
		out	GIFR,temp		

		in	temp,GICR
		sbr	temp,(1 << INT1)		;
		out	GICR,temp			; re-enable external int
	
t2_int_end:	pop	temp
		out	SREG,isr_save
		reti	

;***************************************************************************
;--- 4800 baud aux serial receive support, half duplex, if tx-ing, will ignore rx
;--- comm port support, transition on int1 pin starts us up, then use timer int
int0_isr:	in	isr_save,SREG			; Store Status Register
		push	temp

		ldi	temp, 256 - (aux_bit_reload + (aux_bit_reload/2))
		out	TCNT0,temp			;		
						
		ldi	temp,(1<<TOV0)			;  
		out	TIFR,temp			; to clear any existing timer 0 overflow flag

		in	temp,TIMSK			; get current interupt enable mask
		sbr	temp,(1 << TOIE0)		; 
		out	TIMSK,temp			; enable Timer 0 overflow interrupt

		in	temp,GICR			; get current int enables
		cbr	temp,1<<INT0			; 
		out	GICR,temp			; disable external interrupt

		ldi	temp,1<<aux_busy
		mov	aux_uart_status,temp		; clear bit counter, set bit indicate rx busy

int0_end:	pop	temp
		out	SREG,isr_save			; Restore SREG
		reti


;------------------------------------------------------
;--- timer 0 overflow, send rx_buff bit , or build received bits to bytes depending on status bits 
t0int_isr:	in	isr_save,SREG			;  store SREG
		push	temp

		ldi	temp,256 - aux_bit_reload
		out	tcnt0,temp

		inc	aux_uart_status	

		sbrs	aux_uart_status,aux_txrx	;   if tx bit is cleared, jump to rcv support
		rjmp	aux_rcv_bit	

		sbrc	aux_uart_status,3		;   is bit cnt greater then 7?
		rjmp	aux_tx_stop_bit			;   yes handle stop bit	

		sbrc	aux_char_buff,0			;   if LSB in buffer is 1
		sbi	AUX_PORT,AUX_TXD		;   Set transmit to 1
		sbrs	aux_char_buff,0			;   if LSB in buffer is 0
		cbi	AUX_PORT,AUX_TXD		;   set transmit to 0
		lsr	aux_char_buff			;   shift char buffer right
		rjmp	t0_int_end			;   wait for next, timer overflow 

aux_tx_stop_bit:
		sbi	AUX_PORT,AUX_TXD		;  Set transmit to 1
		sbrs	aux_uart_status,0		;  if bit count is 9 stop bit was sent	 
		rjmp	t0_int_end			;  send stop bit, roll on
	
		rjmp	aux_out				;  all done

aux_rcv_bit:	mov	temp,aux_uart_status
		cbr	temp,1<<aux_busy		; mask off busy bit

		cpi	temp,9				; check for stop bit
		breq	aux_got_char

		clc					; not stop bit, rotate in bit, keep rolling
		sbic	AUX_INP,AUX_RXD			; copy input pin to carry
		sec					;
		ror	aux_char_buff			; shift carry bit into data

 		rjmp	t0_int_end			; wait for nect timer int


aux_got_char:	sbrc	tracking_flags,aux_cr_rcvd	; was last string processed?
		rjmp	aux_out				; no, toss char
		
		mov	temp,aux_char_buff
		cpi	temp,lf				; test for lf
		breq	aux_out				; toss char
	
		cpi	temp,cr				; test for cr
		brne	aux_store_char	

		sbr	tracking_flags,(1 << aux_cr_rcvd) ; set flag to indicate cr received
		rjmp	aux_out

aux_store_char:	lds	temp,_aux_rx_cnt		; get current char count
		cpi	temp,aux_rx_len
		brsh	aux_out				; too long, toss char		

		push	zl				; save registers
		push	zh		
		_init_z	_aux_rx_buffer			; get location

		add	zl,temp
		adc	zh,zero
		st	z,aux_char_buff			; save char
	
		inc	temp				; inc char counter
		sts	_aux_rx_cnt,temp		; save it

		pop	zh				; restore work register	
		pop	zl				; restore work register


aux_out:	in	temp,TIMSK 
		cbr	temp,(1 << TOIE0)		; mask off bit
		out	TIMSK,temp	    		; disable timer 1 int
		
		clr	aux_uart_status			; clear rx busy flag

		ldi	temp,(1<<intf0)			; clear any pending edge flag by writing a one to it
		out	GIFR,temp		

		in	temp,GICR
		sbr	temp,(1 << INT0)		;
		out	GICR,temp			; re-enable external int

	
t0_int_end:	pop	temp
		out	SREG,isr_save
		reti

;--- put char out on aux serial port
putc_aux:	sbrc	aux_uart_status,aux_busy	; wait if isr routine is busy
		rjmp	putc_aux

		in	temp2,GICR
		cbr	temp2,1<<INT0			; 
		out	GICR,temp2			; Disable external interrupt

		ldi	temp2,(1<<aux_busy|1<<aux_txrx|1<<aux_cnt0|1<<aux_cnt1|1<<aux_cnt2|1<<aux_cnt3)
		mov	aux_uart_status,temp2

		ldi	temp2,256 - aux_bit_reload	; initial bit delay
		out	tcnt0,temp2			;
	
		ldi	temp2,(1 << TOV0)		;  
		out	TIFR,temp2			; write a one to clear T/C0 overflow flag (avr weirdness)	

		in	temp2,TIMSK			; get current interupt enable mask
		sbr	temp2,(1 << TOIE0)		; 
		out	TIMSK,temp2			; enable T/C0 overflow interrupt

		mov	aux_char_buff,temp		; put into output buffer

		cbi	AUX_PORT,AUX_TXD		; set txd to 0, start bit
		ret

;-----------------------------------------------------------------
;--- enable aux data reception
en_aux_rx:
		sbi	AUX_PORT,AUX_TXD		; Set transmit to 1 (normal condition)

 		in	temp,MCUCR			; get current mcucr	
 		sbr	temp,1<<ISC01
		cbr	temp,1<<ISC00
		out	MCUCR,temp		 	; set ext int to falling edge mode 	

		clr	aux_uart_status		 	; set to rx mode, not busy

		ldi	temp,(1<<intf0)			; clear edge flag, by writing a one to it (avr weirdness)
		out	GIFR,temp		
			
		in 	temp,GICR			; (new gimsk) get current GICR
		sbr	temp,(1 << INT0)	 	;
		out	GICR,temp		 	; enable external int0 pin
		ret

;-----------------------------------------------------------------
;--- enable wx port data reception
en_wx_rx:	in	temp,MCUCR			; get current mcucr	
 		sbr	temp,1<<ISC11
		cbr	temp,1<<ISC10
		out	MCUCR,temp		 	; set ext int to falling edge mode 	

		clr	wx_uart_status		 	; clear bit counter

		ldi	temp,(1<<INTF1)			; clear edge flag, by writing a one to it (avr weirdness)
		out	GIFR,temp		
			
		in 	temp,GICR			; (new gimsk) get current GICR
		sbr	temp,(1 << INT1)	 	;
		out	GICR,temp		 	; enable external int0 pin
		ret


;***************************************************************************
;--- hardware uart char reception
;--- rs-232 char reception isr, check for errors / discard bad chars and linefeeds
;--- meant for ascii data, binary might cause problems
;--- if cr inc cr_cnt, if ctr-c set ctrl-c flag bit, if enq, set bit
;--- put char in  circ buffer (enq does not go into buffer)  
uart_rx_isr:	in	isr_save,SREG			; store SREG

		push	temp				; work space			
		push	temp2
		push	zh				
		push	zl

		sbic	UCSRA,FE			; skip if framing error
		rjmp	uart_err			; 

		in	temp, UDR			; get rx byte

		sbic	UCSRA,DOR			; skip if overrun error
		rjmp	uart_err			; 

		rcall	int_putc_e			; echo support

		cpi	temp,lf				; discard line feeds
 		breq	uart_isr_end

		cpi	temp,bs				; attempted back space support
 		brne	uart_rx_a	

 
 		lds	temp,_232_rx_buffer
	 	tst	temp
	 	breq	uart_isr_end			; buffer is empty

	 	dec	temp				; one less
		sts	_232_rx_buffer,temp		; save new length	 		
	
		lds	temp,_232_rx_buffer+2		; get tail
		subi	temp,1				; sub 1
		brcc	pc + 2				; check for roll over
	 	ldi	temp,rs232_rx_len		
		sts	_232_rx_buffer+2,temp		; get tail
 
  	
		ldi	temp,' '
		rcall	int_putc_e			; will only send if echo = on
		
		ldi	temp,bs
		rcall	int_putc_e			; will only send if echo = on
		rjmp	uart_isr_end			; bs char was discarded


uart_rx_a:	cpi	temp,cr				; test for cr
		brne	test_enq
		inc	cr_cnt				
		rjmp	uart_store	

test_enq:	cpi	temp,enq			; test for cr
		brne	test_ctrl_c
		sbr	sys_flags,(1 << enq_rcvd)
		rjmp	uart_isr_end			; don't store char	
		
test_ctrl_c:	cpi	temp,ctrl_c			; test for ctrl-c
		brne	pc + 2
		sbr	sys_flags,(1 << ctrl_c_rcvd)	; set flag to indicate ctrl-c received

;---------------------------------------------------------------------
uart_store:	_init_z	_232_rx_buffer			; put on circ buffer
		ldi	temp2,rs232_rx_len		

		rcall	push_circ_z	
		brcs	cts_off				; see if char was stored

		tst	cr_cnt				; char was not stored
		brne	pc + 2
		inc	cr_cnt				; dummy to tell main routine buffer is full

cts_off:	sts	_rs232_timer,zero		; zero char received counter (incs @ 1hz) 

		lds	temp,_232_rx_buffer+0		; see if we are getting near end of buffer
		cpi	temp,rs232_hs_hi
	
		brlo	uart_isr_end
		cts_off
		rjmp	uart_isr_end

uart_err:	in	temp, UDR			; read and discard byte to clear UDR

uart_isr_end:	pop	zl
		pop	zh
		pop	temp2
		pop	temp				; restore
		out	SREG,isr_save			; restore	
		reti

;---------------------------------------------------------------------------
;--- put chars on output buffer from within uart rx isr ,  ints are off
int_putc_e:	sbrs	nv_flags,echo			; echo flag support
		ret	
int_putc:	_init_z	_232_tx_buffer			; put on circ buffer
		ldi	temp2,rs232_tx_len		
		rcall	push_circ_z	
		sbi	UCSRB,UDRIE			; enable udr int 
		ret

;***************************************************************************
;--- hardware uart tx buffer empty
;--- try to get char from circ buffer, send it, else if buffer empty,  turn off int
udr_int:	in	isr_save,SREG			; store SREG
		push	temp			
		push	temp2
		push	zh
		push	zl
	
		_init_z	_232_tx_buffer			; point to buffer
		ldi	temp2,rs232_tx_len		; size of buffer	
		rcall	circ_pop		

		brcc	udr_buff_empty			; if empty

		out	UDR,temp			; send it
		rjmp	udr_int_end
		
udr_buff_empty:	cbi	UCSRB,UDRIE
udr_int_end:	pop	zl				
		pop	zh
		pop	temp2
		pop	temp				; restore
		out	SREG,isr_save			; restore	
		reti


;***************************************************************************
;--- put char (temp) to console via int driven circ buffer
putc:		;wdr				; uncomment these lines to not use the output char buffer
		;sbis	UCSRA,UDRE		; Is UART transmitter ready?
		;rjmp	putc			; If not, wait
		;out	UDR,temp		; Put character to UART
		;ret

		push	temp2			; save 
		push	zl			; save 
		push	zh			; save
		 
		_init_z	_232_tx_buffer		; put on circ buffer
		ldi	temp2,rs232_tx_len

		brie	putc_lp			; if ints are disabled (we were called from an isr)	

		rcall	push_circ_z
		sbi	UCSRB,UDRIE
		rjmp	putc_end

putc_lp:	wdr
		cli				; isr will screw up pointers
		rcall	push_circ_z	
		sei
		sbi	UCSRB,UDRIE		; enable udr int, even if buffer was full enable/re-enable int 
		brcc	putc_lp			; if buffer was full try again

putc_end:	pop	zh			; restore
		pop	zl			; restore
		pop	temp2			; restore
		ret


;***************************************************************************
;--- push temp onto circ buffer pointed to by z
;--- temp2 has buffer length
push_circ_z:	push	temp				; save data

		ld	temp,z				; get size
		cp	temp2,temp			; are we full ?
		breq	push_circ_end			; full up, discard char and exit

		inc	temp				; we will be saving something, so inc
		st	z,temp				; and save

		ldd	temp,z+2			; det the tail pointer
		inc	temp				; bump it
		cp	temp,temp2			; did it wrap? If Temp > length, then yes
		brlo	push_circ_st			; if less than
		breq	push_circ_st			; or equal, it's ok
		ldi	temp,1				; an incremented zero!	

push_circ_st:	std	z+2,temp			; z has size, point to tail, and save

		add	zl,temp				; add the tail pointer to the base
		adc	zh,zero				;  

		pop	temp				; retrieve data
		std	z+2,temp			; the offset gives us data location

		sec
		ret	

push_circ_end:	clc					; nothing stored
		pop	temp				; dummy pop, but return with temp intact
		ret
;***************************************************************************
;--- get char from circ buffer z points to buffer, temp2 = size	
;--- trash z	
circ_pop:	ld	temp,z				; anything in buffer? 
		tst	temp		;
		breq	circ_pop_end_empty		; empty,  nothing to do.
		dec	temp				; dec size
		st	z,temp				; and save it

		ldd	temp,z+1			; get the head pointer
		inc	temp				; bump it
		cp	temp,temp2			; Will it wrap?, temp2 has length

		brlo	circ_pop_a			; If less than temp2, then no
		breq	circ_pop_a			; If equal then no.

		ldi	temp,1				; an incremented zero

circ_pop_a:	std	z+1,temp			; Put the head pointer back
		add	zl,temp				;
		adc	zh,zero				; Now pointing at the data
		ldd	temp,z+2			; Get the data
		sec					; Success
		ret					;

circ_pop_end_empty:
		clc					;
		ret					;

;***************************************************************************
;--- attempt to get  a single char from rs232 buffer  --------
getc_232:	_init_z _232_rx_buffer			; point to buffer	
		ldi	temp2,rs232_rx_len		; size of buffer
		cli
		rcall	circ_pop
		push	temp
		in	temp2,sreg			; save status reg (need to save carry status)
		lds	temp,_232_rx_buffer		; get size

		cpi	temp,rs232_hs_lo		
		brsh	getc_1
		cts_on					; 
getc_1:		pop	temp
		out	sreg,temp2
		reti

;***************************************************************************
;--- copy cr terminated string from rs buffer to null terminated work buffer
get232_str2work:
		_init_y	_work_buff			; set destination	
		ldi	temp3,max_str_len-1 		; size of work buffer, less one for null termination
		rcall	getfrom_232
		st	y,zero				; null terminated	
		ret

;--- copy cr terminated data from rs232 buffer to ax25_buffer, no termination, just return length
get232_str2axbuff:
		_init_y	_ax25_rxtx_buffer
		ldi	temp3,(max_packet_data-2)	; leave room for cr/lf 
	 	
;----------------------------------------------------------------------------
;--- copy cr terminated string from circ buffer @ z to ram  @ y, (don't copy cr)
;--- return length less cr in temp
;--- z must be set to circ buffer, temp2=length of circ buffer 
;--- y points to destination, temp3= max size of linear buffer
getfrom_232:	push	ii				; we need a char counter
		clr	ii				; 

get_str_lp:	rcall	getc_232
		brcc	dec_cr				; no data in buffer

		cpi	temp,cr				; if cr we are done
		brne	get_str_st			; nope, store it

dec_cr:		tst	cr_cnt
		breq	get_str_end			; but don't dec past zero
		
		dec	cr_cnt		
		rjmp 	get_str_end	

get_str_st:	st	y+,temp				; store it

		inc	ii
		cp	ii,temp3			; are we at end of linear buffer ? 
		brsh	get_str_end			; 	

		rjmp	get_str_lp			; loop it

get_str_end:	mov	temp,ii				; return length
		pop	ii
		ret

;***************************************************************************
;--- reset aux rcv buffer (linear), and reset flag bit(s)
flush_aux_buff:	sts	_aux_rx_cnt,zero	
		cbr	tracking_flags,(1<<aux_cr_rcvd)		
		ret
;---------------------------------------------------------------------------
;--- reset wx rcv buffer (linear), and reset flag bit(s)
flush_wx_buff:	sts	_wx_rx_cnt,zero	
		ldi	temp,(0xff ^ (1<<wx_cr_rcvd))
		and	wx_uart_status,temp		; clear bit flag
		ret		
;---------------------------------------------------------------------------
;--- reset rs-232 input buffer (circ), and reset flag bit(s)
flush232_rx_buff:
		cli
		clr	cr_cnt				; no cr rcvd
		cbr	sys_flags,1<<ctrl_c_rcvd | 1<<enq_rcvd ; no ctrl-c or eng
		sts	_232_rx_buffer+0,zero		; null out buffer
		sts	_232_rx_buffer+1,zero
		sts	_232_rx_buffer+2,zero
		cts_on					; enable pc
		reti			
